from amuse.test.amusetest import TestWithMPI
import os
import sys
import numpy
import math

from amuse.community.secularmultiple.interface import SecularMultipleInterface, SecularMultiple

from amuse.units import units,quantities,constants
from amuse.datamodel import Particles

"""
try:
    from matplotlib import pyplot
    HAS_MATPLOTLIB = True
except ImportError:
    HAS_MATPLOTLIB = False
"""
HAS_MATPLOTLIB = False  # disable plotting in the test script

def parse_arguments():
    from amuse.units.optparse import OptionParser
    parser = OptionParser()
    parser.add_option("--test",
                    dest="test",type="int",default=1,
                    help="")

    options, args = parser.parse_args()
    return options.__dict__

class TestSecularMultipleInterface(TestWithMPI):

    def test0(self):
        instance = SecularMultipleInterface()
        instance.stop()

def create_binary(m1,m2,a,e,i,g,h):
    particles = Particles(3)

    for index in range(2):
        particle = particles[index]
        particle.is_binary = False
        if index == 0:
            particle.mass = m1
        else:
            particle.mass = m2
        particle.child1 = None
        particle.child2 = None

    particles[2].is_binary = True
    particles[2].semimajor_axis = a
    particles[2].eccentricity = e
    particles[2].inclination = i
    particles[2].argument_of_pericenter = g
    particles[2].longitude_of_ascending_node = h
    particles[2].child1 = particles[0]
    particles[2].child2 = particles[1]

    return particles

def create_triple(m1,m2,m3,a_in,a_out,e_in,e_out,i_in,i_out,g_in,g_out,h_in,h_out):
    N_bodies = 3
    N_binaries = 2

    m_list = [m1,m2,m3]
    a_list = [a_in,a_out]
    e_list = [e_in,e_out]
    i_list = [i_in,i_out]
    g_list = [g_in,g_out]
    h_list = [h_in,h_out]

    particles = Particles(N_bodies + N_binaries)

    for index in range(N_bodies):
        particle = particles[index]
        particle.is_binary = False
        particle.mass = m_list[index]
        particle.child1 = None
        particle.child2 = None

    for index in range(N_binaries):
        particles[index+N_bodies].is_binary = True
        particles[index+N_bodies].semimajor_axis = a_list[index]
        particles[index+N_bodies].eccentricity = e_list[index]
        particles[index+N_bodies].inclination = i_list[index]
        particles[index+N_bodies].argument_of_pericenter = g_list[index]
        particles[index+N_bodies].longitude_of_ascending_node = h_list[index]

        if index==0:
            particles[index+N_bodies].child1 = particles[0]
            particles[index+N_bodies].child2 = particles[1]
        elif index==1:
            particles[index+N_bodies].child1 = particles[2]
            particles[index+N_bodies].child2 = particles[3]

    return particles

def create_quadruple_triple_single(m1,m2,m3,m4,a_A,a_B,a_C,e_A,e_B,e_C,i_A,i_B,i_C,g_A,g_B,g_C,h_A,h_B,h_C):
    N_bodies = 4
    N_binaries = 3

    m_list = [m1,m2,m3,m4]
    a_list = [a_A,a_B,a_C]
    e_list = [e_A,e_B,e_C]
    i_list = [i_A,i_B,i_C]
    g_list = [g_A,g_B,g_C]
    h_list = [h_A,h_B,h_C]

    particles = Particles(N_bodies + N_binaries)

    for index in range(N_bodies):
        particle = particles[index]
        particle.is_binary = False
        particle.mass = m_list[index]
        particle.child1 = None
        particle.child2 = None

    for index in range(N_binaries):
        particle = particles[index+N_bodies]
        particle.is_binary = True
        particle.semimajor_axis = a_list[index]
        particle.eccentricity = e_list[index]
        particle.inclination = i_list[index]
        particle.argument_of_pericenter = g_list[index]
        particle.longitude_of_ascending_node = h_list[index]

        if index==0:
            particle.child1 = particles[1]
            particle.child2 = particles[0]
        elif index==1:
            particle.child1 = particles[2]
            particle.child2 = particles[4]
        elif index==2:
            particle.child1 = particles[5]
            particle.child2 = particles[3]

    return particles


def create_nested_multiple(N,masses,semimajor_axes,eccentricities,inclinations,arguments_of_pericentre,longitudes_of_ascending_node):
    """
    N is number of bodies
    masses should be N-sized array
    the other arguments should be (N-1)-sized arrays
    """

    N_bodies = N
    N_binaries = N-1

    particles = Particles(N_bodies + N_binaries)

    for index in range(N_bodies):
        particle = particles[index]
        particle.is_binary = False
        particle.mass = masses[index]
        particle.child1 = None
        particle.child2 = None

    previous_binary = particles
    for index in range(N_binaries):
        particle = particles[index+N_bodies]
        particle.is_binary = True
        particle.semimajor_axis = semimajor_axes[index]
        particle.eccentricity = eccentricities[index]
        particle.inclination = inclinations[index]
        particle.argument_of_pericenter = arguments_of_pericentre[index]
        particle.longitude_of_ascending_node = longitudes_of_ascending_node[index]


        if index==0:
            particle.child1 = particles[0]
            particle.child2 = particles[1]
        else:
            particle.child1 = previous_binary
            particle.child2 = particles[index+1]

        previous_binary = particle

    return particles

class TestSecularMultiple(TestWithMPI):
    def test0(self):
        instance = SecularMultiple()
        print(instance.parameters)


    def test1(self):
        """
        test reference system of Naoz et al. (2009)
        """
        particles = create_nested_multiple(3,[1.0|units.MSun, 1.0|units.MJupiter, 40.0|units.MJupiter], [6.0|units.AU,100.0|units.AU], [0.001,0.6], [0.0001,65.0*numpy.pi/180.0],[45.0*numpy.pi/180.0,0.0001],[0.01,0.01])
        binaries = particles[particles.is_binary]

        binaries.include_pairwise_1PN_terms = False

        code = SecularMultiple(redirection='none')
        code.particles.add_particles(particles)

        channel_to_code = particles.new_channel_to(code.particles)
        channel_from_code = code.particles.new_channel_to(particles)


        channel_to_code.copy()
        self.assertEqual(0.6, code.particles[particles.is_binary][1].eccentricity)

        t = 0.0 | units.Myr
        dt = 1.0e-2 | units.Myr
        tend = 3.0e0 | units.Myr

        t_print_array = quantities.AdaptingVectorQuantity()
        a_print_array = quantities.AdaptingVectorQuantity()
        e_print_array = quantities.AdaptingVectorQuantity()
        INCL_print_array = quantities.AdaptingVectorQuantity()
        AP_print_array = quantities.AdaptingVectorQuantity()

        N = 0
        while (t<tend):
            t+=dt
            N+=1
            code.evolve_model(t)
            print('t/Myr = ',code.model_time.value_in(units.Myr))

            channel_from_code.copy()
            t_print_array.append(t)
            a_print_array.append(binaries[0].semimajor_axis)
            e_print_array.append(binaries[0].eccentricity | units.none)
            INCL_print_array.append(binaries[0].inclination_relative_to_parent | units.none)
            AP_print_array.append(binaries[0].argument_of_pericenter | units.none)

        if HAS_MATPLOTLIB == True:
            fig = pyplot.figure(figsize=(16,10))
            plot1 = fig.add_subplot(2,1,1)
            plot2 = fig.add_subplot(2,1,2,yscale="log")

            plot1.plot(t_print_array.value_in(units.Myr),(180.0/numpy.pi)*INCL_print_array.value_in(units.none), color='k')
            plot2.plot(t_print_array.value_in(units.Myr),1.0-e_print_array.value_in(units.none), color='k')

            fontsize = 15
            plot1.set_ylabel("$i$",fontsize=fontsize)
            plot2.set_ylabel("$1-e$",fontsize=fontsize)
            plot2.set_xlabel("$t/\mathrm{Myr}$",fontsize=fontsize)

            pyplot.show()

    def test2(self):
        """
        test 1PN precession in 2-body system
        """
        particles = create_nested_multiple(2,[1.0|units.MSun, 1.0|units.MJupiter], [1.0|units.AU], [0.99], [0.01], [0.01], [0.01])
        binaries = particles[particles.is_binary]

        binaries.include_pairwise_1PN_terms = True

        code = SecularMultiple(redirection='none')
        code.particles.add_particles(particles)

        channel_to_code = particles.new_channel_to(code.particles)
        channel_from_code = code.particles.new_channel_to(particles)

        channel_to_code.copy()

        t = 0.0 | units.Myr
        dt = 1.0e-1 | units.Myr
        tend = 1.0e0 | units.Myr

        t_print_array = quantities.AdaptingVectorQuantity()
        a_print_array = quantities.AdaptingVectorQuantity()
        e_print_array = quantities.AdaptingVectorQuantity()
        AP_print_array = quantities.AdaptingVectorQuantity()

        N = 0
        while (t<tend):
            t+=dt
            N+=1
            code.evolve_model(t)

            channel_from_code.copy()

            print('t/Myr',t.value_in(units.Myr),'omega',binaries[0].argument_of_pericenter | units.none)
            t_print_array.append(t)
            a_print_array.append(binaries[0].semimajor_axis)
            e_print_array.append(binaries[0].eccentricity | units.none)
            AP_print_array.append(binaries[0].argument_of_pericenter | units.none)


        a = binaries[0].semimajor_axis
        e = binaries[0].eccentricity
        M = binaries[0].mass
        rg = constants.G*M/(constants.c**2)
        P = 2.0*numpy.pi*numpy.sqrt(a**3/(constants.G*M))
        t_1PN = (1.0/3.0)*P*(1.0-e**2)*(a/rg)

        if HAS_MATPLOTLIB == True:
            fig = pyplot.figure(figsize=(16,10))
            plot1 = fig.add_subplot(4,1,1)
            plot2 = fig.add_subplot(4,1,2,yscale="log")
            plot3 = fig.add_subplot(4,1,3,yscale="log")
            plot4 = fig.add_subplot(4,1,4,yscale="log")

            plot1.plot(t_print_array.value_in(units.Myr),AP_print_array.value_in(units.none), color='r')
            points = numpy.linspace(0.0,tend.value_in(units.Myr),N)
            AP = 0.01 +2.0*numpy.pi*points/(t_1PN.value_in(units.Myr))
            AP = (AP+numpy.pi)%(2.0*numpy.pi) - numpy.pi ### -pi < AP < pi
            plot1.plot(points,AP,color='g')

            plot2.plot(t_print_array.value_in(units.Myr),numpy.fabs( (AP - AP_print_array.value_in(units.none))/AP ), color='r')
            plot3.plot(t_print_array.value_in(units.Myr),numpy.fabs((a-a_print_array)/a), color='r')
            plot4.plot(t_print_array.value_in(units.Myr),numpy.fabs((e-e_print_array)/e), color='r')

            fontsize = 15
            plot1.set_ylabel("$\omega$",fontsize=fontsize)
            plot2.set_ylabel("$|(\omega_p-\omega)/\omega_p|$",fontsize=fontsize)
            plot3.set_ylabel("$|(a_0-a)/a_0|$",fontsize=fontsize)
            plot4.set_ylabel("$|(e_0-e)/e_0|$",fontsize=fontsize)

            pyplot.show()

    def test3(self):
        """
        test GW emission in 2-body system + collision detection
        """
        particles = create_nested_multiple(2,[1.0|units.MSun, 1.0|units.MJupiter], [0.1|units.AU], [0.994], [0.01], [0.01], [0.01])
        binaries = particles[particles.is_binary]
        stars = particles - binaries
        stars.radius = 0.0001 | units.AU

        binaries.check_for_physical_collision_or_orbit_crossing = True
        binaries.include_pairwise_25PN_terms = True

        code = SecularMultiple(redirection='none')
        code.particles.add_particles(particles)

        channel_to_code = particles.new_channel_to(code.particles)
        channel_from_code = code.particles.new_channel_to(particles)

        channel_to_code.copy()

        t_print_array = quantities.AdaptingVectorQuantity()
        a_print_array = quantities.AdaptingVectorQuantity()
        e_print_array = quantities.AdaptingVectorQuantity()
        AP_print_array = quantities.AdaptingVectorQuantity()

        tend = 1.0 | units.Gyr
        N = 0
        t = 0.0 | units.Myr
        dt = 100.0 | units.Myr
        while (t<tend):
            t+=dt
            N+=1
            code.evolve_model(t)
            flag = code.flag

            if flag == 2:
                print('root found')
                break

            channel_from_code.copy()
            t_print_array.append(t)
            a_print_array.append(binaries[0].semimajor_axis)
            e_print_array.append(binaries[0].eccentricity | units.none)
            AP_print_array.append(binaries[0].argument_of_pericenter | units.none)

            print('e',binaries.eccentricity,'a/AU',binaries.semimajor_axis.value_in(units.AU),'rp/AU',(binaries.semimajor_axis*(1.0-binaries.eccentricity)).value_in(units.AU))

        if HAS_MATPLOTLIB == True:
            fig = pyplot.figure(figsize=(16,10))
            plot1 = fig.add_subplot(2,1,1)
            plot2 = fig.add_subplot(2,1,2,yscale="log")

            plot1.plot(t_print_array.value_in(units.Myr),e_print_array.value_in(units.none), color='r')

            plot2.plot(t_print_array.value_in(units.Myr),a_print_array.value_in(units.AU), color='r')

            fontsize = 15
            plot1.set_ylabel("$e$",fontsize=fontsize)
            plot2.set_ylabel("$a/\mathrm{AU}$",fontsize=fontsize)
            plot2.set_xlabel("$t/\mathrm{Myr}$",fontsize=fontsize)

            pyplot.show()

    def test4(self):
        """
        test tidal friction in 2-body system
        """

        M = 1.0|units.MJupiter
        R = 40.0|units.RJupiter
        m_per = 1.0|units.MSun
        m = m_per
        mu = m*M/(m+M)
        a0 = 0.1 | units.AU
        e0 = 0.3
        P0 = 2.0*numpy.pi*numpy.sqrt(a0**3/(constants.G*(M+m_per)))
        n0 = 2.0*numpy.pi/P0

        aF = a0*(1.0-e0**2)
        nF = numpy.sqrt( constants.G*(M+m_per)/(aF**3) )

        particles = create_nested_multiple(2, [m_per, M], [a0], [e0], [0.01], [0.01], [0.01])
        binaries = particles[particles.is_binary]
        particles[0].radius = 1.0 | units.RSun
        particles[1].radius = R
        particles[1].spin_vec_x = 0.0 | 1.0/units.day
        particles[1].spin_vec_y = 0.0 | 1.0/units.day
        particles[1].spin_vec_z = 4.0e-2 | 1.0/units.day

        k_L = 0.38
        k_AM = k_L/2.0
        rg = 0.25
        tau = 0.66 | units.s

        I = rg*M*R**2
        alpha = I/(mu*a0**2)
        T = R**3/(constants.G*M*tau)
        t_V = 3.0*(1.0 + 1.0/k_L)*T

        particles[1].tides_method = 2
        particles[1].include_tidal_friction_terms = True
        particles[1].include_tidal_bulges_precession_terms = False
        particles[1].include_rotation_precession_terms = False
        particles[1].minimum_eccentricity_for_tidal_precession = 1.0e-8

        particles[1].tides_apsidal_motion_constant = k_AM
        particles[1].tides_viscous_time_scale = t_V
        particles[1].tides_gyration_radius = rg

        tD = M*aF**8/(3.0*k_L*tau*constants.G*m_per*(M+m_per)*R**5)
        particles[2].check_for_physical_collision_or_orbit_crossing = True

        code = SecularMultiple(redirection='none')
        code.particles.add_particles(particles)

        code.parameters.relative_tolerance = 1.0e-14

        channel_to_code = particles.new_channel_to(code.particles)
        channel_from_code = code.particles.new_channel_to(particles)

        channel_to_code.copy()

        t = 0.0 | units.Myr
        dt = 1.0e-5 | units.Myr
        tend = 1.0e-4 | units.Myr

        t_print_array = quantities.AdaptingVectorQuantity()
        a_print_array = quantities.AdaptingVectorQuantity()
        n_print_array = quantities.AdaptingVectorQuantity()
        e_print_array = quantities.AdaptingVectorQuantity()
        AP_print_array = quantities.AdaptingVectorQuantity()
        spin_print_array = quantities.AdaptingVectorQuantity()

        while (t<tend):
            t+=dt
            code.evolve_model(t)
            print('flag',code.flag,t,'a/AU',binaries[0].semimajor_axis,'e',binaries[0].eccentricity)

            channel_from_code.copy()

            t_print_array.append(t)
            a_print_array.append(binaries[0].semimajor_axis)
            n_print_array.append(numpy.sqrt(constants.G*(M+m)/(binaries[0].semimajor_axis**3)))
            e_print_array.append(binaries[0].eccentricity | units.none)
            AP_print_array.append(binaries[0].argument_of_pericenter | units.none)
            spin_print_array.append( numpy.sqrt( particles[1].spin_vec_x**2 + particles[1].spin_vec_y**2 + particles[1].spin_vec_z**2) )

            binaries = particles[particles.is_binary]
            bodies = particles - binaries
            print('S_x',bodies.spin_vec_x.value_in(1.0/units.day))
            print('S_y',bodies.spin_vec_y.value_in(1.0/units.day))
            print('S_z',bodies.spin_vec_z.value_in(1.0/units.day))
            print('='*50)

        if HAS_MATPLOTLIB == True:
            fig = pyplot.figure()
            fontsize=12

            N_p = 4
            plot1 = fig.add_subplot(N_p,1,1)
            plot1.plot(t_print_array.value_in(units.Myr),a_print_array.value_in(units.AU), color='r')
            plot1.set_ylabel("$a/\mathrm{AU}$",fontsize=fontsize)

            plot2 = fig.add_subplot(N_p,1,2)
            plot2.plot(t_print_array.value_in(units.Myr),e_print_array.value_in(units.none),color='k')
            plot2.set_ylabel("$e$",fontsize=fontsize)

            plot3 = fig.add_subplot(N_p,1,3,yscale="log")

            plot3.plot(t_print_array.value_in(units.Myr),a_print_array.value_in(units.AU)*(1.0-e_print_array.value_in(units.none)**2),color='k')
            plot3.axhline(y = a0.value_in(units.AU)*(1.0 - e0**2), color='k')
            plot3.set_ylabel("$a(1-e^2)/\mathrm{AU}$",fontsize=fontsize)

            plot4 = fig.add_subplot(N_p,1,4)
            plot4.plot(t_print_array.value_in(units.Myr),spin_print_array/n_print_array)
            plot4.set_ylabel("$\Omega/n$",fontsize=fontsize)

            plot4.set_xlabel("$t/\mathrm{Myr}$",fontsize=fontsize)

            pyplot.show()

    def test5(self):
        """
        test precession due to tidal bulges
        """

        M = 1.0|units.MJupiter
        R = 1.0|units.RJupiter
        m_per = 1.0|units.MSun
        a0 = 30.0 | units.AU
        e0 = 0.999
        P0 = 2.0*numpy.pi*numpy.sqrt(a0**3/(constants.G*(M+m_per)))
        n0 = 2.0*numpy.pi/P0

        particles = create_nested_multiple(2, [m_per, M], [a0], [e0], [0.01], [0.01], [0.01])
        binaries = particles[particles.is_binary]
        particles[0].radius = 1.0 | units.RSun
        particles[1].radius = R

        k_L = 0.41
        k_AM = k_L/2.0

        particles[1].tides_method = 0
        particles[1].include_tidal_friction_terms = False
        particles[1].include_tidal_bulges_precession_terms = True
        particles[1].include_rotation_precession_terms = False
        particles[1].minimum_eccentricity_for_tidal_precession = 1.0e-5
        particles[1].tides_apsidal_motion_constant = k_AM
        particles[1].tides_gyration_radius = 0.25
        
        code = SecularMultiple(redirection='none')
        code.particles.add_particles(particles)

        channel_to_code = particles.new_channel_to(code.particles)
        channel_from_code = code.particles.new_channel_to(particles)

        channel_to_code.copy()

        t = 0.0 | units.Myr
        dt = 1.0e1 | units.Myr
        tend = 1.0e2 | units.Myr

        t_print_array = quantities.AdaptingVectorQuantity()
        a_print_array = quantities.AdaptingVectorQuantity()
        e_print_array = quantities.AdaptingVectorQuantity()
        AP_print_array = quantities.AdaptingVectorQuantity()

        g_dot_TB = (15.0/8.0)*n0*(8.0+12.0*e0**2+e0**4)*(m_per/M)*k_AM*pow(R/a0,5.0)/pow(1.0-e0**2,5.0)
        t_TB = 2.0*numpy.pi/g_dot_TB

        N=0
        while (t<tend):
            t+=dt
            code.evolve_model(t)
            print('flag',code.flag,t,binaries[0].semimajor_axis,binaries[0].eccentricity)

            channel_from_code.copy()

            t_print_array.append(t)
            a_print_array.append(binaries[0].semimajor_axis)
            e_print_array.append(binaries[0].eccentricity | units.none)
            AP_print_array.append(binaries[0].argument_of_pericenter | units.none)

            N+=1
        if HAS_MATPLOTLIB == True:
            fig = pyplot.figure(figsize=(16,10))
            plot1 = fig.add_subplot(4,1,1)
            plot2 = fig.add_subplot(4,1,2,yscale="log")
            plot3 = fig.add_subplot(4,1,3,yscale="log")
            plot4 = fig.add_subplot(4,1,4,yscale="log")

            plot1.plot(t_print_array.value_in(units.Myr),AP_print_array.value_in(units.none), color='r')
            points = numpy.linspace(0.0,tend.value_in(units.Myr),N)
            AP = 0.01 +2.0*numpy.pi*points/(t_TB.value_in(units.Myr))
            AP = (AP+numpy.pi)%(2.0*numpy.pi) - numpy.pi ### -pi < AP < pi
            plot1.plot(points,AP,color='g')

            plot2.plot(t_print_array.value_in(units.Myr),numpy.fabs( (AP - AP_print_array.value_in(units.none))/AP ), color='r')
            plot3.plot(t_print_array.value_in(units.Myr),numpy.fabs((a0-a_print_array)/a0), color='r')
            plot4.plot(t_print_array.value_in(units.Myr),numpy.fabs((e0-e_print_array)/e0), color='r')

            fontsize = 15
            plot1.set_ylabel("$\omega$",fontsize=fontsize)
            plot2.set_ylabel("$|(\omega_p-\omega)/\omega_p|$",fontsize=fontsize)
            plot3.set_ylabel("$|(a_0-a)/a_0|$",fontsize=fontsize)
            plot4.set_ylabel("$|(e_0-e)/e_0|$",fontsize=fontsize)

            pyplot.show()

    def test6(self):
        """
        test precession due to rotation
        """

        M = 1.0|units.MJupiter
        R = 1.5|units.RJupiter
        m_per = 1.0|units.MSun
        a0 = 30.0 | units.AU
        e0 = 0.999
        P0 = 2.0*numpy.pi*numpy.sqrt(a0**3/(constants.G*(M+m_per)))
        n0 = 2.0*numpy.pi/P0

        aF = a0*(1.0-e0**2)
        nF = numpy.sqrt( constants.G*(M+m_per)/(aF**3) )

        particles = create_nested_multiple(2, [m_per, M], [a0], [e0], [1.0e-5], [1.0e-5], [1.0e-5])
        binaries = particles[particles.is_binary]
        particles[0].radius = 1.0 | units.RSun
        particles[1].radius = R
        particles[1].spin_vec_x = 0.0 | 1.0/units.day
        particles[1].spin_vec_y = 0.0 | 1.0/units.day
        Omega_PS0 = n0*(33.0/10.0)*pow(a0/aF,3.0/2.0)
        particles[1].spin_vec_z = Omega_PS0


        k_L = 0.51
        k_AM = k_L/2.0
        rg = 0.25
        particles[1].tides_method = 1
        particles[1].include_tidal_friction_terms = False
        particles[1].include_tidal_bulges_precession_terms = False
        particles[1].include_rotation_precession_terms = True
        particles[1].minimum_eccentricity_for_tidal_precession = 1.0e-5
        particles[1].tides_apsidal_motion_constant = k_AM
        particles[1].tides_gyration_radius = rg

        code = SecularMultiple(redirection='none')
        code.particles.add_particles(particles)

        channel_to_code = particles.new_channel_to(code.particles)
        channel_from_code = code.particles.new_channel_to(particles)

        channel_to_code.copy()

        t = 0.0 | units.Myr
        dt = 1.0e1 | units.Myr
        tend = 1.0e2 | units.Myr

        t_print_array = quantities.AdaptingVectorQuantity()
        a_print_array = quantities.AdaptingVectorQuantity()
        e_print_array = quantities.AdaptingVectorQuantity()
        AP_print_array = quantities.AdaptingVectorQuantity()

        Omega_vec = [particles[1].spin_vec_x,particles[1].spin_vec_y,particles[1].spin_vec_z]
        Omega = numpy.sqrt(Omega_vec[0]**2 + Omega_vec[1]**2 + Omega_vec[2]**2)
        print('Omega/n',Omega/n0)

        g_dot_rot = n0*(1.0 + m_per/M)*k_AM*pow(R/a0,5.0)*(Omega/n0)**2/((1.0-e0**2)**2)
        t_rot = 2.0*numpy.pi/g_dot_rot
        print('t_rot/Myr',t_rot.value_in(units.Myr))

        N=0
        while (t<tend):
            t+=dt
            code.evolve_model(t)
            print('flag',code.flag,t,binaries[0].semimajor_axis,binaries[0].eccentricity)

            channel_from_code.copy()

            t_print_array.append(t)
            a_print_array.append(binaries[0].semimajor_axis)
            e_print_array.append(binaries[0].eccentricity | units.none)
            AP_print_array.append(binaries[0].argument_of_pericenter | units.none)

            N+=1
        if HAS_MATPLOTLIB == True:
            fig = pyplot.figure(figsize=(16,10))
            plot1 = fig.add_subplot(4,1,1)
            plot2 = fig.add_subplot(4,1,2,yscale="log")
            plot3 = fig.add_subplot(4,1,3,yscale="log")
            plot4 = fig.add_subplot(4,1,4,yscale="log")

            plot1.plot(t_print_array.value_in(units.Myr),AP_print_array.value_in(units.none), color='r')
            points = numpy.linspace(0.0,tend.value_in(units.Myr),N)
            AP = 0.01 +2.0*numpy.pi*points/(t_rot.value_in(units.Myr))
            AP = (AP+numpy.pi)%(2.0*numpy.pi) - numpy.pi ### -pi < AP < pi
            plot1.plot(points,AP,color='g')

            plot2.plot(t_print_array.value_in(units.Myr),numpy.fabs( (AP - AP_print_array.value_in(units.none))/AP ), color='r')
            plot3.plot(t_print_array.value_in(units.Myr),numpy.fabs((a0-a_print_array)/a0), color='r')
            plot4.plot(t_print_array.value_in(units.Myr),numpy.fabs((e0-e_print_array)/e0), color='r')

            fontsize = 15
            plot1.set_ylabel("$\omega$",fontsize=fontsize)
            plot2.set_ylabel("$|(\omega_p-\omega)/\omega_p|$",fontsize=fontsize)
            plot3.set_ylabel("$|(a_0-a)/a_0|$",fontsize=fontsize)
            plot4.set_ylabel("$|(e_0-e)/e_0|$",fontsize=fontsize)

            pyplot.show()

    def test7(self):
        """
        test collision detection in 3-body system
        """

        particles = create_nested_multiple(3,[1.0|units.MSun, 1.2|units.MSun, 0.9|units.MSun], [1.0|units.AU, 100.0|units.AU], [0.1, 0.5], [0.01, 80.0*numpy.pi/180.0], [0.01, 0.01], [0.01, 0.01])
        binaries = particles[particles.is_binary]
        stars = particles - binaries

        binaries.check_for_physical_collision_or_orbit_crossing = True
        stars.radius = 0.03 | units.AU

        code = SecularMultiple(redirection='none')
        code.particles.add_particles(particles)

        channel_to_code = particles.new_channel_to(code.particles)
        channel_from_code = code.particles.new_channel_to(particles)

        channel_to_code.copy()

        t = 0.0 | units.Myr
        dt = 1.0e-2 | units.Myr
        tend = 1.0e-1 | units.Myr

        t_print_array = quantities.AdaptingVectorQuantity()
        a_print_array = quantities.AdaptingVectorQuantity()
        e_print_array = quantities.AdaptingVectorQuantity()

        while (t<tend):
            t+=dt
            code.evolve_model(t)
            flag = code.flag
            channel_from_code.copy()

            print('secular_breakdown_has_occurred',binaries.secular_breakdown_has_occurred)
            print('dynamical_instability_has_occurred',binaries.dynamical_instability_has_occurred)
            print('physical_collision_or_orbit_crossing_has_occurred',binaries.physical_collision_or_orbit_crossing_has_occurred)
            print('minimum_periapse_distance_has_occurred',binaries.minimum_periapse_distance_has_occurred)
            print('RLOF_at_pericentre_has_occurred',binaries.RLOF_at_pericentre_has_occurred)

            if flag == 2:
                print('root found')
                break
            print('t_end',code.model_time.value_in(units.Myr))

            t_print_array.append(t)
            a_print_array.append(binaries[0].semimajor_axis)
            e_print_array.append(binaries[0].eccentricity | units.none)

        if HAS_MATPLOTLIB == True:
            fig = pyplot.figure()
            plot = fig.add_subplot(2,1,1)
            plot.plot(t_print_array.value_in(units.Myr),e_print_array.value_in(units.none))

            plot = fig.add_subplot(2,1,2)
            plot.plot(t_print_array.value_in(units.Myr),a_print_array.value_in(units.AU)*(1.0-e_print_array.value_in(units.none)))
            plot.axhline(y = stars[0].radius.value_in(units.AU) + stars[1].radius.value_in(units.AU),color='k')

            pyplot.show()

    def test8(self):
        """
        test minimum periapse occurrence
        """

        particles = create_nested_multiple(3,[1.0|units.MSun, 1.2|units.MSun, 0.9|units.MSun], [1.0|units.AU, 100.0|units.AU], [0.1, 0.5], [0.01, 80.0*numpy.pi/180.0], [0.01, 0.01], [0.01, 0.01])
        binaries = particles[particles.is_binary]
        stars = particles - binaries

        binaries.check_for_minimum_periapse_distance = True
        rp_min = 0.1 | units.AU
        binaries.check_for_minimum_periapse_distance_value = rp_min

        code = SecularMultiple(redirection='none')
        code.particles.add_particles(particles)

        channel_to_code = particles.new_channel_to(code.particles)
        channel_from_code = code.particles.new_channel_to(particles)

        channel_to_code.copy()

        t = 0.0 | units.Myr
        dt = 5.0e-3 | units.Myr
        tend = 1.0e-1 | units.Myr

        t_print_array = quantities.AdaptingVectorQuantity()
        a_print_array = quantities.AdaptingVectorQuantity()
        e_print_array = quantities.AdaptingVectorQuantity()

        while (t<tend):
            t+=dt
            code.evolve_model(t)
            flag = code.flag
            channel_from_code.copy()

            print('secular_breakdown_has_occurred',binaries.secular_breakdown_has_occurred)
            print('dynamical_instability_has_occurred',binaries.dynamical_instability_has_occurred)
            print('physical_collision_or_orbit_crossing_has_occurred',binaries.physical_collision_or_orbit_crossing_has_occurred)
            print('minimum_periapse_distance_has_occurred',binaries.minimum_periapse_distance_has_occurred)
            print('RLOF_at_pericentre_has_occurred',binaries.RLOF_at_pericentre_has_occurred)

            if flag == 2:
                print('root found')
                break
            print('t_end',code.model_time.value_in(units.Myr))

            t_print_array.append(t)
            a_print_array.append(binaries[0].semimajor_axis)
            e_print_array.append(binaries[0].eccentricity | units.none)

        if HAS_MATPLOTLIB == True:
            fig = pyplot.figure()
            plot = fig.add_subplot(2,1,1)
            plot.plot(t_print_array.value_in(units.Myr),e_print_array.value_in(units.none))

            plot = fig.add_subplot(2,1,2)
            plot.plot(t_print_array.value_in(units.Myr),a_print_array.value_in(units.AU)*(1.0-e_print_array.value_in(units.none)))
            plot.axhline(y = rp_min.value_in(units.AU),color='k')

            plot.set_ylim(0.0,1.0)

            pyplot.show()

if __name__ in ('__main__','__plot__'):
    testSecularMultiple = TestSecularMultiple()

    cmd_options = parse_arguments()

    if cmd_options["test"] == 1:
        testSecularMultiple.test1()
    if cmd_options["test"] == 2:
        testSecularMultiple.test2()
    if cmd_options["test"] == 3:
        testSecularMultiple.test3()
    if cmd_options["test"] == 4:
        testSecularMultiple.test4()
    if cmd_options["test"] == 5:
        testSecularMultiple.test5()
    if cmd_options["test"] == 6:
        testSecularMultiple.test6()
    if cmd_options["test"] == 7:
        testSecularMultiple.test7()
    if cmd_options["test"] == 8:
        testSecularMultiple.test8()




from amuse.test.amusetest import TestWithMPI

from amuse.community.interface import gd
from amuse.community.interface.gd import GravitationalDynamicsInterface
from amuse.rfi.tools import create_definition
from amuse.rfi.tools.create_definition import CodeDocStringProperty
from amuse.rfi.core import LegacyFunctionSpecification
class TestGravitationalDynamics(TestWithMPI):
    def test1(self):
        x = GravitationalDynamicsInterface()
        
        function = GravitationalDynamicsInterface.new_particle
        specification = function.specification
        self.assertTrue(specification.description.startswith("Define a new particle"))
        
    def test2(self):
        specification = LegacyFunctionSpecification()
        specification.name ='test'
        specification.addParameter('one','d',specification.IN, 'first parameter')
        specification.description = 'Example function'
        
        x = create_definition.CreateDescriptionOfAFunctionSpecification()
        x.specification = specification
        x.start()
        
        self.assertTrue(x.out.string.find('void test(float64 one)') > 0)
        self.assertTrue(x.out.string.find('Example function') >= 0)
        self.assertTrue(x.out.string.find(':param one:') > 0)


        
    def test3(self):
        specification = LegacyFunctionSpecification()
        specification.name ='test'
        specification.addParameter('one','d',specification.IN, 'first parameter')
        specification.result_type = 'i'
        specification.result_doc = 'an integer'
        specification.description = 'Example function'
        
        x = create_definition.CreateDescriptionOfAFunctionSpecification()
        x.specification = specification
        x.start()
        
        self.assertTrue(x.out.string.find('int32 test(float64 one)') > 0)
        self.assertTrue(x.out.string.find(':returns:') > 0)
        
    
        
    def test4(self):
        specification = LegacyFunctionSpecification()
        specification.name ='test'
        specification.addParameter('one','d',specification.IN, 'first parameter')
        specification.result_type = 'i'
        specification.result_doc = 'an integer'
        specification.description = 'Example function'
        
        x = create_definition.CreateFortranStub()
        x.specification = specification
        x.start()
        
        self.assertTrue(x.out.string.find('FUNCTION test(one)') >= 0)
        self.assertTrue(x.out.string.find('END FUNCTION') > 0)
        self.assertTrue(x.out.string.find('DOUBLE PRECISION :: one') > 0)
        
    def test5(self):
        class WithLegacyDocStringProperty(object):
            
            def __init__(self):
                "orignal doc"
                pass
            
            @property
            def specification(self):
                specification = LegacyFunctionSpecification()
                specification.name ='test'
                specification.addParameter('one','d',specification.IN, 'first parameter')
                specification.result_type = 'i'
                specification.result_doc = 'an integer'
                specification.description = 'Example function'
                return specification
                
            __doc__ = CodeDocStringProperty()
            
            
        self.assertEqual("orignal doc", WithLegacyDocStringProperty.__doc__)
        instance = WithLegacyDocStringProperty()
        instance_documentation =  WithLegacyDocStringProperty().__doc__
        self.assertTrue(instance_documentation.find('FUNCTION test(one)') >= 0)
        
    def test6(self):
        print("Testing description of Legacy Function with output parameter")
        specification = LegacyFunctionSpecification()
        specification.name ='test'
        specification.addParameter('one','d',specification.OUT, 'first parameter')
        specification.result_type = 'i'
        specification.result_doc = 'an integer'
        specification.description = 'Example function'
        
        x = create_definition.CreateDescriptionOfAFunctionSpecification()
        x.specification = specification
        x.start()
        self.assertTrue(x.out.string.find('int32 test(float64 * one)') > 0)
        self.assertTrue(x.out.string.find(':returns:') > 0)
        
    def test7(self):
        print("Testing __str__ of Legacy Function")
        specification = LegacyFunctionSpecification()
        specification.name ='test'
        specification.addParameter('one','f',specification.IN, 'first parameter, type: float')
        specification.addParameter('two','d',specification.OUT, 'second parameter, type double')
        specification.result_type = 'i'
        specification.result_doc = 'an integer'
        specification.description = 'Example function'
        self.assertEqual(str(specification),"function: int test(float one)\noutput: double two, int __result")
        



from amuse.test import amusetest

from amuse.support.interface import InCodeComponentImplementation

from amuse.datamodel.indexing import *
from amuse.datamodel import indexing

class TestIndexing(amusetest.TestCase):
    def test1(self):
        self.assertEqual(2, number_of_dimensions_after_index(3, 1))
        self.assertEqual(3, number_of_dimensions_after_index(3, numpy.s_[0:3]))
        self.assertEqual(1, number_of_dimensions_after_index(3, combine_indices(3,2)))
        self.assertEqual(0, number_of_dimensions_after_index(3, combine_indices(combine_indices(3,2),1)))
        self.assertEqual(3, indexing.number_of_dimensions_after_index(3, numpy.s_[1:2,...,...])) 
        self.assertEqual(3, indexing.number_of_dimensions_after_index(3, numpy.s_[1:2,:,:])) 
        
    def test2(self):
        a = numpy.arange(12).reshape(3,4)
        self.assertEqual(a[combine_indices(0,1)], a[0][1])
        self.assertEqual(a[combine_indices(1,0)], a[1][0])
        self.assertTrue(numpy.all(a[combine_indices(1,numpy.s_[0:2])] == a[1][0:2]))
        indirect = combine_indices(0,1)
        self.assertEqual(number_of_dimensions(a, indirect), 0)
        
        
    def test3(self):
        a = numpy.arange(12).reshape(3,4)
        self.assertTrue(a[combine_indices(numpy.s_[0:2],0)].shape, a[0:2][0].shape)
        self.assertTrue(numpy.all(a[combine_indices(numpy.s_[0:2],0)] ==  a[0:2][0]))

    def test4(self):
        a = numpy.arange(12).reshape(3,4)
        direct =  a[1][:]
        indirect = a[combine_indices(1, indexing.normalize_slices(a[1].shape,numpy.s_[:]))]
        self.assertEqual(indirect.shape, direct.shape)
        self.assertTrue(numpy.all(indirect ==  direct))
        
    def test5(self):
        a = numpy.arange(12).reshape(3,4)
        direct =  a[0:2][:]
        indirect = a[combine_indices(numpy.s_[0:2],indexing.normalize_slices(a[0:2].shape,numpy.s_[:]))]
        self.assertEqual(indirect.shape, direct.shape)
        self.assertTrue(numpy.all(indirect ==  direct))

    def test6(self):
        a = numpy.arange(12).reshape(3,4)
        direct =  a[1:3][1:]
        indirect = a[combine_indices(numpy.s_[1:3],indexing.normalize_slices(a[1:3].shape,numpy.s_[1:]))]
        self.assertEqual(indirect.shape, direct.shape)
        self.assertTrue(numpy.all(indirect ==  direct))

    def test7(self):
        a = numpy.arange(30).reshape(6,5)
        direct =  a[1:5:2][1:]
        indirect = a[combine_indices(numpy.s_[1:5:2],indexing.normalize_slices(a[1:5:2].shape,numpy.s_[1:]))]
        self.assertEqual(indirect.shape, direct.shape)
        self.assertTrue(numpy.all(indirect ==  direct))
        
    def test8(self):
        a = numpy.arange(30)
        direct =  a[2:14:3][1:5:2]
        indirect = a[combine_indices(numpy.s_[2:14:3],numpy.s_[1:5:2])]
        self.assertEqual(indirect.shape, direct.shape)
        self.assertTrue(numpy.all(indirect ==  direct))
        
    
    def test9(self):
        a = numpy.arange(100)
        for s in range(0,40):
            for e in range(40,101):
                for step in range(1,5):
                    direct =  a[s:e:step][1:5:2]
                    indirect = a[combine_indices(numpy.s_[s:e:step], 
                      indexing.normalize_slices(a[s:e:step].shape,numpy.s_[1:5:2]))]
                    self.assertEqual(indirect.shape, direct.shape)
                    self.assertTrue(numpy.all(indirect ==  direct))
    
    def test10(self):
        a = numpy.arange(60).reshape(5,6,2)
        direct =  a[3][2][1]
        indirect = a[combine_indices(combine_indices(3,2),1)]
        self.assertEqual(indirect, direct)
        
    def test11(self):
        a = numpy.arange(60).reshape(5,6,2)
        direct =  a[3]
        indirect = a[combine_indices(3,Ellipsis)]
        self.assertEqual(indirect.shape, direct.shape)
        self.assertTrue(numpy.all(indirect ==  direct))
    
    def test12(self):
        self.assertEqual((1,4,2), indexing.shape_after_index((5,4,2), numpy.s_[1:2,...,...])) 
        self.assertEqual((1,4,2), indexing.shape_after_index((5,4,2), numpy.s_[1:2,:,:])) 
        self.assertEqual((2,4,2), indexing.shape_after_index((5,4,2), numpy.s_[1:3,...,...])) 
        self.assertEqual((2,4,2), indexing.shape_after_index((5,4,2), numpy.s_[1:3,:,:])) 
        self.assertEqual((2,1,2), indexing.shape_after_index((5,4,2), numpy.s_[1:3,2:3,...])) 
        self.assertEqual((2,1,2), indexing.shape_after_index((5,4,2), numpy.s_[1:3,2:3,:])) 
        
    
    def xtest13(self):
        combined_indices = combine_indices(numpy.s_[1:3],numpy.s_[:])
        self.assertEqual(combined_indices, numpy.s_[1:3:1])
        combined_indices = combine_indices(numpy.s_[:], numpy.s_[1:3])
        self.assertEqual(combined_indices, numpy.s_[1:3:1])
        combined_indices = combine_indices((numpy.s_[:], numpy.s_[:]), (numpy.s_[1:3], numpy.s_[1:2]))
        self.assertEqual(combined_indices,(numpy.s_[1:3:1], numpy.s_[1:2:1]))
        
        combined_indices = combine_indices((numpy.s_[0:2], numpy.s_[:]), (numpy.s_[1:3], numpy.s_[1:2]))
        self.assertEqual(combined_indices,(numpy.s_[1:2:1], numpy.s_[1:2:1]))
        
        
    def test14(self):
        self.assertEqual((5,4,2), indexing.shape_after_index((5,4,2), numpy.s_[:10,...,...]))
        self.assertEqual((5,4,2), indexing.shape_after_index((5,4,2), numpy.s_[...,:10,...]))
        self.assertEqual((4,4,2), indexing.shape_after_index((5,4,2), numpy.s_[1:10,...,...]))
        self.assertEqual((1,4,2), indexing.shape_after_index((5,4,2), numpy.s_[-1:,...,...]))
        self.assertEqual((2,4,2), indexing.shape_after_index((5,4,2), numpy.s_[-2:,...,...]))
        self.assertEqual((1,4,2), indexing.shape_after_index((5,4,2), numpy.s_[-2:-1,...,...]))
        self.assertEqual((5,4,2), indexing.shape_after_index((5,4,2), numpy.s_[-10:,...,...]))
        

    
    def test15(self):
        a = numpy.arange(6).reshape(2,3)
        indices = numpy.asarray([[True, False, True],[True,False,True]])
        direct =  a[indices][list([1,3])]
        combined = combine_indices(indices,[1,3])
        indirect = a[combined]
        self.assertEqual(indirect, direct)
        
    
    def test16(self):
        self.assertEqual((4,), indexing.shape_after_index((2,3), [[True, False, True],[True,False,True]]))
        self.assertEqual((1,3), indexing.shape_after_index((2,3), [True, False]))
    
    def test17(self):
        a = numpy.arange(6).reshape(2,3)
        indices = numpy.asarray([True,False])
        direct =  a[indices, 1:][0,1:]
        combined = combine_indices(indexing.normalize_slices(a.shape,numpy.s_[indices,1:]),
            indexing.normalize_slices(a[indices,1:].shape,numpy.s_[0,1:]))
        indirect = a[combined]
        self.assertEqual(indirect, direct)
        
    def test18(self):
        a = numpy.arange(6).reshape(2,3)
        indices = numpy.asarray([True,False])
        direct =  a[indices, 1:][0]
        combined = combine_indices(numpy.s_[indices,1:],0)
        indirect = a[combined]
        self.assertEqual(indirect, direct)
        
    def test19(self):
        a = numpy.arange(6).reshape(2,3)
        indices = numpy.asarray([True,False])
        direct =  a[:1, 1:][0]
        combined = combine_indices(numpy.s_[:1,1:],0)
        indirect = a[combined]
        self.assertEqual(indirect, direct)

    def test20(self):
        combined=combine_indices( slice(1, 199, None), slice(3, 5, None) )
        self.assertEqual(combined,slice(4,6,1))
        
    def test21(self):
        shape=shape_after_index((200,), slice(4, 6, 1))
        self.assertEqual(shape,(2,))

    def xtest22(self):
        tiny=list(range(2))
        small=list(range(10))
        big=list(range(1000))
        
        # combining slicings w negative stops not possible! e.g. ((7,-1),(2,3),(9,10,1))
        # (without normalize)
        slicings=[ ((9,19),(5,9,2),(14,18,2)),
                   ((7,19,2),(5,9,1),(17,19,2)),
                   ((1,None),(1,10),(2,11,1)),
                   ((7,None),(1,10),(8,17,1)),
                   ((None,12),(3,5),(3,5,1)),
                   ((None,12),(3,15),(3,12,1)),
                   ((None,None),(3,5),(3,5,1)),
                   ((None,None),(3,15),(3,15,1)),
                   ((None,None),(None,15),(0,15,1)),
                   ((None,None),(None,None),(0,None,1)),
                   ((9,None),(None,None),(9,None,1)),
                   ((9,None),(6,None),(15,None,1)),
                   ((9,None),(None,40),(9,49,1)),
                   ((1,None),(None,40),(1,41,1)),
                   ((9,16),(None,40),(9,16,1)),
                   ((1,16),(None,40),(1,16,1)),
                   ((49,16),(None,40),(16,16,1)),
                   ((41,66),(None,40),(41,66,1)),
                 ]
        
        for t1,t2,t3 in slicings:
          s1=slice(*t1)
          s2=slice(*t2)
          s3=slice(*t3)
          self.assertEqual(combine_slices(s1,s2),t3)
          self.assertTrue(tiny[s1][s2]==tiny[s3])
          self.assertTrue(small[s1][s2]==small[s3])
          self.assertTrue(big[s1][s2]==big[s3])
        
    def xtest23(self):
        import random
        random.seed(123456)
        tiny=list(range(2))
        small=list(range(20))
        big=list(range(2000))
        
        Ntest=1000
        start0=[random.randint(0,20) for x in range(Ntest)]        
        stop0=[random.randint(15,50) for x in range(Ntest)]        
        step0=[random.randint(1,3) for x in range(Ntest)]        
        start1=[random.randint(0,10) for x in range(Ntest)]        
        stop1=[random.randint(5,25) for x in range(Ntest)]        
        step1=[random.randint(1,3) for x in range(Ntest)]        
        
        slicings=[]
        for x in zip(start0,stop0,step0,start1,stop1,step1):
          slicings.append(((x[0],x[1],x[2]),(x[3],x[4],x[5])))
        
        for t1,t2 in slicings:
          s1=slice(*t1)
          s2=slice(*t2)
          t3=combine_slices(s1,s2)
          s3=slice(*t3)
          self.assertTrue(tiny[s1][s2]==tiny[s3])
          self.assertTrue(small[s1][s2]==small[s3])
          self.assertTrue(big[s1][s2]==big[s3])

    def test24(self):
        import random
        random.seed(123456)
        tiny=list(range(2))
        small=list(range(20))
        big=list(range(2000))
        
        Ntest=1000
        stop0=[random.randint(0,20) for x in range(Ntest)]        
        start0=[random.randint(15,50) for x in range(Ntest)]        
        step0=[random.randint(-3,-1) for x in range(Ntest)]        
        start1=[random.randint(0,10) for x in range(Ntest)]        
        stop1=[random.randint(5,25) for x in range(Ntest)]        
        step1=[random.randint(1,3) for x in range(Ntest)]        
        
        slicings=[]
        for x in zip(start0,stop0,step0,start1,stop1,step1):
          slicings.append(((x[0],x[1],x[2]),(x[3],x[4],x[5])))
        
        for t1,t2 in slicings:
          s1=slice(*t1)
          s2=slice(*t2)

          t3=combine_slices(normalize_slices(len(tiny),s1),normalize_slices(len(tiny[s1]),s2))
          s3=slice(*t3)
          self.assertTrue(tiny[s1][s2]==tiny[s3])

          t3=combine_slices(normalize_slices(len(small),s1),normalize_slices(len(small[s1]),s2))
          s3=slice(*t3)
          self.assertTrue(small[s1][s2]==small[s3])
          t3=combine_slices(normalize_slices(len(big),s1),normalize_slices(len(big[s1]),s2))
          s3=slice(*t3)
          self.assertTrue(big[s1][s2]==big[s3])

    def test25(self):
        import random
        random.seed(123456)
        tiny=list(range(2))
        small=list(range(20))
        big=list(range(2000))
        
        Ntest=1000
        stop0=[random.randint(0,20) for x in range(Ntest)]        
        start0=[random.randint(15,50) for x in range(Ntest)]        
        step0=[random.randint(-3,-1) for x in range(Ntest)]        
        stop1=[random.randint(0,10) for x in range(Ntest)]        
        start1=[random.randint(5,25) for x in range(Ntest)]        
        step1=[random.randint(-3,-1) for x in range(Ntest)]        
        
        slicings=[]
        for x in zip(start0,stop0,step0,start1,stop1,step1):
            slicings.append(((x[0],x[1],x[2]),(x[3],x[4],x[5])))
        
        for t1,t2 in slicings:
            s1=slice(*t1)
            s2=slice(*t2)

            t3=combine_slices(normalize_slices(len(tiny),s1),normalize_slices(len(tiny[s1]),s2))
            s3=slice(*t3)
            #~ print(s1,s2,s3)
            self.assertTrue(tiny[s1][s2]==tiny[s3])

            t3=combine_slices(normalize_slices(len(small),s1),normalize_slices(len(small[s1]),s2))
            s3=slice(*t3)
            self.assertTrue(small[s1][s2]==small[s3])
            t3=combine_slices(normalize_slices(len(big),s1),normalize_slices(len(big[s1]),s2))
            s3=slice(*t3)
            self.assertTrue(big[s1][s2]==big[s3])

    def test26(self):
        oned=numpy.zeros(5)
        threed=numpy.zeros((4,5,6))
        for index in [0,[1],[1,2],[[1,2],[2,3]],[[2]],[[0,1]]]:
          i=numpy.array(index)
          self.assertEqual(len(oned[i].shape), number_of_dimensions_after_index(1, i ))
        for index in [0,[1],[1,2],[[1,2],[2,3]],[[2]],[[2,1]]]:
          i=numpy.array(index)
          self.assertEqual(len(threed[i].shape), number_of_dimensions_after_index(3, i ))

    def test27(self):
        oned=numpy.zeros(5)
        threed=numpy.zeros((4,5,6))
        for index in [0,[1],[1,2],[[1,2],[2,3]],[[2]],[[0,1]]]:
          i=numpy.array(index)
          self.assertEqual(oned[i].shape, shape_after_index(oned.shape, i ))
        for index in [0,[1],[1,2],[[1,2],[2,3]],[[2]],[[2,1]],[[[[0],[1],[1]]]]]:
          i=numpy.array(index)
          self.assertEqual(threed[i].shape, shape_after_index(threed.shape, i ))

    def test28(self):
        twod=numpy.zeros((5,6))
        threed=numpy.zeros((4,5,6))
        for _i,_j in [([0],[1]),([0,2],[1,3]),([0,2],[1,3]),([[0,1],[1,2]],[[2,3],[3,4]])]:
          i=numpy.array(_i)
          j=numpy.array(_j)
          self.assertEqual(len(twod[i,j].shape), number_of_dimensions_after_index(2, (i,j) ))
        for _i,_j in [([0],[1]),([0,2],[1,3]),([0,2],[1,3]),([[0,1],[1,2]],[[2,3],[3,4]])]:
          i=numpy.array(_i)
          j=numpy.array(_j)
          self.assertEqual(len(threed[i,j].shape), number_of_dimensions_after_index(3, (i,j) ))

    def test29(self):
        twod=numpy.zeros((5,6))
        for _i,_j in [([0],[1]),([0,2],[1,3]),([0,2],[1,3]),([[0,1],[1,2]],[[2,3],[3,4]])]:
          i=numpy.array(_i)
          j=numpy.array(_j)
          self.assertEqual(twod[i,j].shape, shape_after_index(twod.shape, (i,j) ))

        threed=numpy.zeros((4,5,6))
        for _i,_j in [([0],[1]),([0,2],[1,3]),([0,2],[1,3]),([[0,1],[1,2]],[[2,3],[3,4]])]:
          i=numpy.array(_i)
          j=numpy.array(_j)
          self.assertEqual(threed[i,j].shape, shape_after_index(threed.shape, (i,j) ))

        fourd=numpy.zeros((4,5,6,7))
        for _i,_j in [([0],[1]),([0,2],[1,3])]:
          i=numpy.array(_i)
          j=numpy.array(_j)
          self.assertEqual(fourd[i,j].shape, shape_after_index(fourd.shape, (i,j) ))
        for _i,_j in [([0,2],[1,3])]:
          i=numpy.array(_i)
          j=numpy.array(_j)
          self.assertEqual(fourd[i,Ellipsis,j].shape, shape_after_index(fourd.shape, (i,Ellipsis,j) ))
        for _i,_j in [([0,2],[1,3])]:
          i=numpy.array(_i)
          j=numpy.array(_j)
          self.assertEqual(fourd[i,slice(None),j].shape, shape_after_index(fourd.shape, (i,slice(None),j) ))


        
class TestSplitOverDimensions(amusetest.TestCase):
    
    def test1(self):
        dimension_values = [
            [3,4,5,6],
            ['a','b','c']
        ]
        split_dimension_values = indexing.split_numpy_index_over_dimensions(0, dimension_values)
        self.assertEqual(len(split_dimension_values) ,2)
        self.assertEqual(split_dimension_values[0], 3)
        self.assertEqual(split_dimension_values[1], ['a','b','c'])
       
    def test2(self):
        dimension_values = [
            [3,4,5,6],
            ['a','b','c']
        ]
        split_dimension_values = indexing.split_numpy_index_over_dimensions((1,2), dimension_values)
        self.assertEqual(len(split_dimension_values) ,2)
        self.assertEqual(split_dimension_values[0], 4)
        self.assertEqual(split_dimension_values[1], 'c')
    
    def test3(self):
        dimension_values = [
            [3,4,5,6],
            ['a','b','c']
        ]
        split_dimension_values = indexing.split_numpy_index_over_dimensions(slice(0,2), dimension_values)
        self.assertEqual(len(split_dimension_values) ,2)
        self.assertEqual(split_dimension_values[0], [3,4])
        self.assertEqual(split_dimension_values[1], ['a','b','c'])
        
    def test4(self):
        dimension_values = [
            [0, 1, 2, 3, 4, 5, 6, 7, 8, ]
        ]
        split_dimension_values = indexing.split_numpy_index_over_dimensions(slice(1,7,2), dimension_values)
        self.assertEqual(len(split_dimension_values) ,1)
        self.assertEqual(split_dimension_values[0], [1, 3, 5])
        
    
    def test5(self):
        dimension_values = [
            [0, 1, 2, 3, 4, 5, 6, 7, 8,9 ]
        ]
        split_dimension_values = indexing.split_numpy_index_over_dimensions(slice(-2,10), dimension_values)
        self.assertEqual(split_dimension_values[0], [8, 9])
        split_dimension_values = indexing.split_numpy_index_over_dimensions(slice(-3,3,-1), dimension_values)
        self.assertEqual(split_dimension_values[0], [7, 6, 5, 4])
    
    
    def test6(self):
        dimension_values = [
            [0, 1, 2, 3, 4, 5, 6, 7, 8,9 ]
        ]
        split_dimension_values = indexing.split_numpy_index_over_dimensions(slice(5,None), dimension_values)
        self.assertEqual(split_dimension_values[0], [5, 6, 7, 8, 9])
    
    
    def test7(self):
        dimension_values = [
            [0, 1],
            [0, 1, 2],
            [0]
        ]
        split_dimension_values = indexing.split_numpy_index_over_dimensions(slice(1,2), dimension_values)
        self.assertEqual(split_dimension_values[0], [1])
        self.assertEqual(split_dimension_values[1], [0,1,2])
        self.assertEqual(split_dimension_values[2], [0])
    
    
    def test8(self):
        dimension_values = [
            [0, 1],
            [0, 1, 2],
            [0]
        ]
        split_dimension_values = indexing.split_numpy_index_over_dimensions((Ellipsis,0), dimension_values)
        self.assertEqual(split_dimension_values[0], [0,1])
        self.assertEqual(split_dimension_values[1], [0,1,2])
        self.assertEqual(split_dimension_values[2], 0)
        
        split_dimension_values = indexing.split_numpy_index_over_dimensions((slice(None),slice(None),0), dimension_values)
        self.assertEqual(split_dimension_values[0], [0,1])
        self.assertEqual(split_dimension_values[1], [0,1,2])
        self.assertEqual(split_dimension_values[2], 0)
    
        split_dimension_values = indexing.split_numpy_index_over_dimensions((Ellipsis,0, Ellipsis), dimension_values)
        self.assertEqual(split_dimension_values[0], [0,1])
        self.assertEqual(split_dimension_values[1], 0)
        self.assertEqual(split_dimension_values[2], [0])



from amuse.test import amusetest

import os.path
from amuse import io
from amuse.units import units

class TestsForTicket118(amusetest.TestCase):
    
    def test1(self):
        filename = os.path.join(os.path.dirname(__file__), 'FinalSnapshot.out')
        set = io.read_set_from_file(filename, 'dyn')
        
        self.assertEqual(len(set), 10)
        
    def test2(self):
        filename = os.path.join(os.path.dirname(__file__), 'FinalSnapshot.out')
        
        root = io.read_set_from_file(filename, 'dyn', return_children=False)
        
        self.assertFalse(root is None)
        
    def test3(self):
        filename = os.path.join(os.path.dirname(__file__), 'FinalSnapshot.out')
        set = io.read_set_from_file(filename, 'dyn')
        print(set)
        print("set[0].parent.mass", set[0].parent.mass)
        self.assertAlmostRelativeEquals(0.000227826766314251919 * 617.75586357299929284496, set[9].mass.value_in(units.MSun), 12)
        self.assertAlmostRelativeEquals(617.75586357299929284496 * 0.953575109205781479,  set[0].parent.mass.value_in(units.MSun), 12)
        #self.assertTrue(False)



from amuse.test import amusetest
from amuse.units import units, nbody_system, constants
from amuse.ic.isotropic_cloud import new_isotropic_cloud
from amuse.ext.orbital_elements import orbital_elements_from_binary
from amuse.datamodel import Particles

class TestSphericalIsotropicCloud(amusetest.TestCase):
  
  def test1(self):
    cloud = new_isotropic_cloud(2, m_cloud=0.|units.MSun)
    self.assertEqual(len(cloud),2)
    self.assertEqual(cloud.mass.sum(), 0.|units.MSun)
    
  def test2(self):
    m_star = 0.666|units.MSun
    a_min=666.|units.AU
    a_max=6666.|units.AU
    q_min=6.|units.AU
    cloud = new_isotropic_cloud(66,
                                m_star=m_star,
                                a_min=a_min,
                                a_max=a_max,
                                q_min=q_min)
    binary = Particles(1)
    binary[0].mass = m_star
    binary[0].position = (0.,0.,0.) | units.AU
    binary[0].velocity = (0.,0.,0.) | units.kms
    for comet in cloud:
      binary.add_particle(comet)
      mass1, mass2, semimajor_axis, eccentricity, true_anomaly, inclination, long_asc_node, arg_per = \
        orbital_elements_from_binary(binary, G=constants.G)
      print(mass1, mass2, semimajor_axis, eccentricity, true_anomaly, inclination, long_asc_node, arg_per)
      self.assertTrue( a_min < semimajor_axis < a_max )
      self.assertTrue( q_min < semimajor_axis*(1.-eccentricity) )
      binary.remove_particle(comet)



import sys
import os
import numpy.random

from amuse.test import amusetest
from amuse.units import units, nbody_system
from amuse.ext.boss_bodenheimer import bb79_cloud

numpy.random.seed(1234567)

class BossBodenheimerTests(amusetest.TestCase):
    def test1(self):
        numpy.random.seed(1234)
        mc=bb79_cloud(targetN=1000).result
        self.assertEqual(len(mc),1000)
        ek=mc.kinetic_energy()
        ep=mc.potential_energy(G=nbody_system.G)
        eth=mc.thermal_energy()
        self.assertAlmostEqual(eth/ep, -0.25, 2)
        self.assertAlmostEqual(ek/ep, -0.2, 2)

    def test2(self):
        numpy.random.seed(1234)
        convert=nbody_system.nbody_to_si(1. | units.MSun,3.2e16| units.cm)
        mc=bb79_cloud(targetN=1000,convert_nbody=convert).result
        self.assertEqual(len(mc),1000)
        ek=mc.kinetic_energy()
        ep=mc.potential_energy()
        eth=mc.thermal_energy()
        self.assertAlmostEqual(eth/ep, -0.25, 2)
        self.assertAlmostEqual(ek/ep, -0.2, 2)




import numpy

from amuse.test import amusetest
from amuse.units import units, nbody_system, constants

from amuse.ext.rotating_bridge import Rotating_Bridge, inertial_to_rotating, rotating_to_inertial

from amuse.ext.composition_methods import *

from amuse.datamodel import Particles

class drift_without_gravity(object):
    """
    This class evolves the motion of the test particles (no gravity) 
    """
    def __init__(self, particles, time= 0 |units.Myr):
        self.particles = particles
        self.model_time = time
    def evolve_model(self, t_end):
        dt = t_end - self.model_time
        self.particles.position += self.particles.velocity*dt
        self.model_time = t_end
    @property
    def potential_energy(self):
        return quantities.zero
    @property 
    def kinetic_energy(self):
        return (0.5*self.particles.mass*self.particles.velocity.lengths()**2).sum()

class TestRotatingBridge(amusetest.TestCase):
    def test1(self):
        p0=Particles(1)
        p0.position = [0.0, 1.0, 0.0]
        p0.velocity = [1.0, 0.0, 0.0]
        
        pos1=p0.position.copy()
        vel1=p0.velocity.copy()
        
        omega=1.
        N=0.25
        dt=0.01
        method = SPLIT_6TH_SS_M13

        tend=N*2*numpy.pi/omega
        dt=dt*2*numpy.pi/omega

        pr=inertial_to_rotating(0,omega, p0)
        
        drift=drift_without_gravity(pr, time=0)
        sys=Rotating_Bridge(omega, timestep=dt, method=method)
        sys.add_system(drift)
        
        sys.evolve_model(tend)
        
        pi=rotating_to_inertial(tend, omega,pr)
        
        pos2=pi.position.copy()
        
        
        self.assertAlmostEqual(sys.model_time,tend)
        self.assertAlmostEqual(pos1+vel1*tend,pos2,12)
        
        pi.velocity=-pi.velocity
        
        pr=inertial_to_rotating(0,-omega,pi)
        drift=drift_without_gravity(pr,time=0)
        sys=Rotating_Bridge(-omega, timestep=dt,method=method)
        sys.add_system(drift)
        
        sys.evolve_model(tend)        

        pi=rotating_to_inertial(tend,-omega,pr)
        
        pos3=pi.position.copy()
        
        self.assertAlmostEqual(pos1,pos3,12)

    def test2(self):
        p0=Particles(1)
        p0.position = [0.0, 1.0, 0.0] | units.m
        p0.velocity = [1.0, 0.0, 0.0] | units.m/units.s
        
        pos1=p0.position.copy()
        vel1=p0.velocity.copy()
        
        omega=1. | units.s**-1
        N=0.25
        dt=0.01
        method = SPLIT_6TH_SS_M13

        tend=N*2*numpy.pi/omega
        dt=dt*2*numpy.pi/omega

        pr=inertial_to_rotating(0*dt,omega, p0)
        
        drift=drift_without_gravity(pr, time=0 | units.s)
        sys=Rotating_Bridge(omega, timestep=dt, method=method)
        sys.add_system(drift)
        
        sys.evolve_model(tend)
        
        pi=rotating_to_inertial(tend, omega,pr)
        
        pos2=pi.position.copy()
        
        
        self.assertAlmostEqual(sys.model_time,tend)
        self.assertAlmostEqual(pos1+vel1*tend,pos2,12)
        
        pi.velocity=-pi.velocity
        
        pr=inertial_to_rotating(0*dt,-omega,pi)
        drift=drift_without_gravity(pr,time=0| units.s)
        sys=Rotating_Bridge(-omega, timestep=dt,method=method)
        sys.add_system(drift)
        
        sys.evolve_model(tend)        

        pi=rotating_to_inertial(tend,-omega,pr)
        
        pos3=pi.position.copy()
        
        self.assertAlmostEqual(pos1,pos3,12)



from amuse.test.amusetest import TestWithMPI
import os
import sys
import numpy
import math

from amuse.community.hacs64.interface import Hacs64Interface, hacs64

from amuse.units import nbody_system
from amuse.units import units
from amuse import datamodel
from amuse.ic import plummer
try:
    from matplotlib import pyplot
    HAS_MATPLOTLIB = True
except ImportError:
    HAS_MATPLOTLIB = False


class TestHacs64Interface(TestWithMPI):

    def test0(self):
        instance = Hacs64Interface()
        instance.stop()
    
    def test1(self):
        instance = Hacs64Interface()
        instance.initialize_code()
        instance.commit_parameters()
    
        res1 = instance.new_particle(mass = 11.0, radius = 2.0, x = 0.0, y = 0.0, z = 0.0, vx = 0.0, vy = 0.0, vz = 0.0)
        res2 = instance.new_particle(mass = 21.0, radius = 5.0, x = 10.0, y = 0.0, z = 0.0, vx = 10.0, vy = 0.0, vz = 0.0)
        
        self.assertEqual(0, res1['index_of_the_particle'])
        self.assertEqual(1, res2['index_of_the_particle'])
        
        error = instance.commit_particles()
        self.assertEqual(0, error)
        retrieved_state1 = instance.get_state(0)
        retrieved_state2 = instance.get_state(1)
        self.assertEqual(0, retrieved_state1['__result'])
        self.assertEqual(0, retrieved_state2['__result'])
        self.assertEqual(11.0,  retrieved_state1['mass'])
        self.assertEqual(21.0,  retrieved_state2['mass'])
        self.assertEqual(0.0,  retrieved_state1['x'])
        self.assertEqual(10.0,  retrieved_state2['x'])
        self.assertEqual(5.0,  retrieved_state2['radius'])
    
        instance.cleanup_code()
        instance.stop()

    def test2(self):
        instance = Hacs64Interface()
        instance.initialize_code()
        instance.commit_parameters()

        for i in [1, 2, 3]:
            temp_particle = instance.new_particle(mass = i, radius = 1.0, x = i * 1.0, y = 0.0, z = 0.0, vx = 0.0, vy = 0.0, vz = 0.0)
            self.assertEqual(i-1, temp_particle['index_of_the_particle'])
            
        error = instance.commit_particles()
        self.assertEqual(0, error)
        error = instance.delete_particle(1)
        self.assertEqual(0, error)
        error = instance.recommit_particles()
        self.assertEqual(0, error)
      
        self.assertEqual(2, instance.get_number_of_particles()['number_of_particles'])
        
        self.assertEqual(0, instance.get_index_of_first_particle()['index_of_the_particle'])
        self.assertEqual(2, instance.get_index_of_next_particle(0)['index_of_the_next_particle'])
        self.assertEqual(0, instance.get_index_of_next_particle(0)['__result'])
        self.assertEqual(-1, instance.get_index_of_next_particle(1)['__result'])
        self.assertEqual(1, instance.get_index_of_next_particle(2)['__result'])
        instance.stop()
        
    def test3(self):
        instance = Hacs64Interface()
        instance.initialize_code()
        self.assertEqual(0, instance.set_nmax(10))
        self.assertEqual((10,0), tuple(instance.get_nmax()))
        self.assertEqual(0, instance.set_dtmax(4))
        self.assertEqual((4,0), tuple(instance.get_dtmax()))
        self.assertEqual(0, instance.set_eps2(2))
        self.assertEqual((2,0), tuple(instance.get_eps2()))
        
        self.assertEqual(0,instance.commit_parameters())
        
        self.assertEqual(-1, instance.set_nmax(11))
        self.assertEqual((10,0), tuple(instance.get_nmax()))
        self.assertEqual(-1, instance.set_dtmax(6))
        self.assertEqual((4,0), tuple(instance.get_dtmax()))
        
        instance.cleanup_code() 
        instance.stop()
 
    def test4(self):
        instance = Hacs64Interface()
        instance.initialize_code()
        self.assertEqual(0, instance.set_h2max(0.1))
        self.assertEqual((0.1,0), tuple(instance.get_h2max()))
        
        self.assertEqual((0.14,0), tuple(instance.get_eta_reg()))
        self.assertEqual(0, instance.set_eta_reg(0.4))
        self.assertEqual((0.4,0), tuple(instance.get_eta_reg()))
        
        self.assertEqual((0.8,0), tuple(instance.get_eta_irr()))
        self.assertEqual(0, instance.set_eta_irr(0.6))
        self.assertEqual((0.6,0), tuple(instance.get_eta_irr()))
        
        self.assertEqual(0,instance.commit_parameters())
        
        instance.cleanup_code() 
        instance.stop()
        
    def test5(self):
        instance = Hacs64Interface()
        instance.initialize_code()
        instance.commit_parameters()
        
        instance.new_particle([10,20],[0,1],[0,0], [0,0], [0,0], [0,0], [0,0],[1,1])
        instance.commit_particles()
        retrieved_state = instance.get_state(0)
        
        self.assertEqual(10.0,  retrieved_state['mass'])
        self.assertEqual(1, retrieved_state['radius'])
    
        retrieved_state = instance.get_state([0,1])
        self.assertEqual(20.0,  retrieved_state['mass'][1])
        self.assertEqual(instance.get_number_of_particles()['number_of_particles'], 2)
        instance.cleanup_code() 
        instance.stop()

    def test6(self):
        instance = Hacs64Interface()
        instance.initialize_code()
        instance.commit_parameters()
        
        instance.new_particle([10,10],[-1,1],[0,0], [0,0], [0,0], [0,0], [0,0],[1,1])
        instance.commit_particles()
        
        self.assertFalse(hasattr(instance, "get_potential_at_point"))
        #retr, error = instance.get_potential_at_point(0.01, 0,0,0)
        #@print retr
        #self.assertEqual(error, -1) # TODO: Not IMPLEMENTED yet
        
        #self.assertEqual(retr['phi'], -20.0)
        instance.cleanup_code()
        instance.stop()
       
    def test7(self):
        instance = Hacs64Interface()
        instance.initialize_code()
        instance.set_eps2(0.1 * 0.1)
        instance.commit_parameters()
        id1,errorcode = instance.new_particle(mass = 10.0, radius = 1.0, x = 0.0, y = 0.0, z = 0.0, vx = 0.0, vy = 0.0, vz = 0.0)
        id2,errorcode = instance.new_particle(mass = 10.0, radius = 1.0, x = 2.0, y = 0.0, z = 0.0, vx = 10.0, vy = 0.0, vz = 0.0)
        
        instance.commit_particles()
        potential, errorcode = instance.get_potential(id1)
        self.assertEqual(errorcode, 0)
        self.assertAlmostRelativeEquals(potential,  -10.0 / numpy.sqrt(2.0**2 + 0.1**2), 8)
        
        potential, errorcode = instance.get_potential(id2)
        self.assertEqual(errorcode, 0)
        self.assertAlmostRelativeEquals(potential,  -10.0 / numpy.sqrt(2.0**2 + 0.1**2), 8)
        
        total_potential, errorcode = instance.get_potential_energy()
        potentials, errorcode = instance.get_potential([id1, id2])
        instance.stop()
        
        self.assertAlmostRelativeEquals(total_potential, numpy.sum(potentials * [10.0, 10.0]) / 2.0)
        

    def test8(self):
        instance = Hacs64Interface()
        instance.initialize_code()
        instance.set_eps2(0)
        instance.commit_parameters()
        id1,errorcode = instance.new_particle(mass = 10.0, radius = 1.0, x = 0.0, y = 0.0, z = 0.0, vx = 0.0, vy = 0.0, vz = 0.0)
        id2,errorcode = instance.new_particle(mass = 1.0, radius = 1.0, x = 2.0, y = 0.0, z = 0.0, vx = 10.0, vy = 0.0, vz = 0.0)
        
        instance.commit_particles()
        potential, errorcode = instance.get_potential(id1)
        self.assertEqual(errorcode, 0)
        self.assertAlmostRelativeEquals(potential,  -1.0 / numpy.sqrt(2.0**2), 8)
        total_potential, errorcode = instance.get_potential_energy()
        potentials, errorcode = instance.get_potential([id1, id2])
        instance.stop()
        
        self.assertAlmostRelativeEquals(total_potential, numpy.sum(potentials * [10.0, 1.0]) / 2.0)
        
    
    def test9(self):
        print("Test HermiteInterface evolve_model")
        instance = Hacs64Interface()
        self.assertEqual(0, instance.initialize_code())
        #self.assertEquals(0, instance.set_eta_irr(0.14))
        #self.assertEquals(0, instance.set_eta_reg(0.001))
        self.assertEqual(0, instance.commit_parameters())
        
        # Set up an equal-mass binary on a circular orbit:
        self.assertEqual([0, 0], list(instance.new_particle(0.5,  0.5, 0, 0,  0, 0.5, 0, 0.01).values()))
        self.assertEqual([1, 0], list(instance.new_particle(0.5, -0.5, 0, 0,  0,-0.5, 0, 0.01).values()))
        self.assertEqual(0, instance.commit_particles())
        self.assertEqual(0, instance.evolve_model(math.pi))
        for result, expected in zip(list(instance.get_position(0).values()), [-0.5, -0.007, 0.0, 0]):
            self.assertAlmostEqual(result, expected, 3)
        for result, expected in zip(list(instance.get_position(1).values()), [0.5, 0.007, 0.0, 0]):
            self.assertAlmostEqual(result, expected, 3)
        
        self.assertEqual(0, instance.evolve_model(2 * math.pi))
        for result, expected in zip(list(instance.get_position(0).values()), [0.5, 0.0147, 0.0, 0]):
            self.assertAlmostEqual(result, expected, 3)
        for result, expected in zip(list(instance.get_position(1).values()), [-0.5, -0.0147, 0.0, 0]):
            self.assertAlmostEqual(result, expected, 3)
        
        self.assertEqual(0, instance.cleanup_code())
        instance.stop()
    


class _TestHacs64(TestWithMPI):
    def new_system_of_sun_and_earth(self):
        stars = datamodel.Stars(2)
        sun = stars[0]
        sun.mass = units.MSun(1.0)
        sun.position = units.m(numpy.array((0.0,0.0,0.0)))
        sun.velocity = units.ms(numpy.array((0.0,0.0,0.0)))
        sun.radius = units.RSun(1.0)

        earth = stars[1]
        earth.mass = units.kg(5.9736e24)
        earth.radius = units.km(6371) 
        earth.position = units.km(numpy.array((149.5e6,0.0,0.0)))
        earth.velocity = units.ms(numpy.array((0.0,29800,0.0)))
        
        return stars
        
    
    def test1(self):
        convert_nbody = nbody_system.nbody_to_si(1.0 | units.MSun, 149.5e6 | units.km)
    
        instance = Hermite(convert_nbody)
        instance.initialize_code()
        instance.parameters.epsilon_squared = 0.0 | units.AU**2
        instance.dt_dia = 5000
        
        stars = self.new_system_of_sun_and_earth()
        earth = stars[1]
                
        instance.particles.add_particles(stars)
        
        instance.evolve_model(365.0 | units.day)
        instance.particles.copy_values_of_all_attributes_to(stars)
        
        position_at_start = earth.position.value_in(units.AU)[0]
        position_after_full_rotation = earth.position.value_in(units.AU)[0]
        self.assertAlmostEqual(position_at_start, position_after_full_rotation, 6)
        
        instance.evolve_model(365.0 + (365.0 / 2) | units.day)
        
        instance.particles.copy_values_of_all_attributes_to(stars)
        position_after_half_a_rotation = earth.position.value_in(units.AU)[0]
        self.assertAlmostEqual(-position_at_start, position_after_half_a_rotation, 2)
                
        instance.evolve_model(365.0 + (365.0 / 2) + (365.0 / 4)  | units.day)
        
        instance.particles.copy_values_of_all_attributes_to(stars)
        position_after_half_a_rotation = earth.position.value_in(units.AU)[1]
        self.assertAlmostEqual(-position_at_start, position_after_half_a_rotation, 3)
        
        instance.cleanup_code()
        
        instance.stop()
        

    def test2(self):
        convert_nbody = nbody_system.nbody_to_si(1.0 | units.MSun, 149.5e6 | units.km)
    
        instance = Hermite(convert_nbody)
        instance.initialize_code()
        instance.parameters.epsilon_squared = 0.0 | units.AU**2
        instance.dt_dia = 5000
        
        stars = self.new_system_of_sun_and_earth()
        earth = stars[1]
        instance.particles.add_particles(stars)
    
        for x in range(1,2000,10):
            instance.evolve_model(x | units.day)
            instance.particles.copy_values_of_all_attributes_to(stars)
            stars.savepoint()
        
        if HAS_MATPLOTLIB:
            figure = pyplot.figure()
            plot = figure.add_subplot(1,1,1)
            
            x_points = earth.get_timeline_of_attribute("x")
            y_points = earth.get_timeline_of_attribute("y")
            
            x_points_in_AU = [t_x[1].value_in(units.AU) for t_x in x_points]
            y_points_in_AU = [t_x1[1].value_in(units.AU) for t_x1 in y_points]
            
            plot.scatter(x_points_in_AU,y_points_in_AU, color = "b", marker = 'o')
            
            plot.set_xlim(-1.5, 1.5)
            plot.set_ylim(-1.5, 1.5)
               
            
            test_results_path = self.get_path_to_results()
            output_file = os.path.join(test_results_path, "instance-earth-sun2.svg")
            figure.savefig(output_file)
        
        
        
        instance.cleanup_code()
        instance.stop()

    def test3(self):
        convert_nbody = nbody_system.nbody_to_si(1.0 | units.MSun, 149.5e6 | units.km)

        instance = Hermite(convert_nbody)
        instance.initialize_code()
        instance.parameters.epsilon_squared = 0.00001 | units.AU**2
        instance.dt_dia = 5000
        
        stars = datamodel.Stars(2)
        star1 = stars[0]
        star2 = stars[1]

        star1.mass = units.MSun(1.0)
        star1.position = units.AU(numpy.array((-1.0,0.0,0.0)))
        star1.velocity = units.AUd(numpy.array((0.0,0.0,0.0)))
        star1.radius = units.RSun(1.0)

        star2.mass = units.MSun(1.0)
        star2.position = units.AU(numpy.array((1.0,0.0,0.0)))
        star2.velocity = units.AUd(numpy.array((0.0,0.0,0.0)))
        star2.radius = units.RSun(100.0)
        
        instance.particles.add_particles(stars)
    
        for x in range(1,2000,10):
            instance.evolve_model(x | units.day)
            instance.particles.copy_values_of_all_attributes_to(stars)
            stars.savepoint()
        
        instance.stop()
    
    def test4(self):
        convert_nbody = nbody_system.nbody_to_si(5.0 | units.kg, 10.0 | units.m)

        instance = Hermite(convert_nbody)
        instance.initialize_code()
        
        particles = datamodel.Particles(2)
        self.assertEqual(len(instance.particles), 0)
        
        particles.mass = [15.0, 30.0] | units.kg
        particles.radius =  [10.0, 20.0] | units.m
        particles.position = [[10.0, 20.0, 30.0], [20.0, 40.0, 60.0]] | units.m
        particles.velocity = [[0.0, 0.0, 0.0], [0.0, 0.0, 0.0]] | units.m / units.s

        
        instance.particles.add_particles(particles)
        self.assertEqual(len(instance.particles), 2)
        
        instance.particles.mass =  [17.0, 33.0] | units.kg
        
        
        self.assertEqual(instance.get_mass(0), 17.0| units.kg) 
        self.assertEqual(instance.get_mass(1), 33.0| units.kg)  
        instance.stop()

    def test5(self):
        convert_nbody = nbody_system.nbody_to_si(5.0 | units.kg, 10.0 | units.m)

        instance = Hermite(convert_nbody)
        instance.initialize_code()
        
        particles = datamodel.Particles(2)
        self.assertEqual(len(instance.particles), 0)
        
        particles.mass = [15.0, 30.0] | units.kg
        particles.radius =  [10.0, 20.0] | units.m
        particles.position = [[10.0, 20.0, 30.0], [20.0, 40.0, 60.0]] | units.m
        particles.velocity = [[0.0, 0.0, 0.0], [0.0, 0.0, 0.0]] | units.m / units.s

        
        instance.particles.add_particles(particles)
        self.assertEqual(len(instance.particles), 2)
        instance.set_state(1, 16|units.kg, 20.0|units.m, 
                           20.0|units.m, 40.0|units.m, 60.0|units.m, 
                           1.0|units.ms, 1.0|units.ms, 1.0|units.ms)
        
        curr_state =  instance.get_state(1)
        instance.stop()
        
        self.assertEqual(curr_state[0], 16|units.kg, 8)
    
    def test6(self):
        print("Test6: Testing Hermite parameters")
        convert_nbody = nbody_system.nbody_to_si(1.0 | units.yr, 1.0 | units.AU)
        instance = Hermite(convert_nbody)
        
        value = instance.get_eps2()
        self.assertEqual(0.0 | units.AU**2 , value)
        self.assertAlmostEqual(0.0 | units.AU**2, instance.parameters.epsilon_squared, in_units=units.AU**2)
        for x in [0.01, 0.1, 0.2]:
            instance.parameters.epsilon_squared = x | units.AU**2
            self.assertAlmostEqual(x | units.AU**2, instance.parameters.epsilon_squared, in_units=units.AU**2)
        
        value = instance.get_dt_param()
        self.assertEqual(0.03, value)
        self.assertAlmostEqual(0.03, instance.parameters.dt_param, in_units=units.none)
        for x in [0.001, 0.01, 0.1]:
            instance.parameters.dt_param = x
            self.assertAlmostEqual(x, instance.parameters.dt_param, in_units=units.none)
        
        value = instance.get_dt_dia()
        self.assertAlmostEqual(1.0 | units.yr, value)
        self.assertAlmostEqual(1.0 | units.yr, instance.parameters.dt_dia, in_units=units.yr)
        for x in [0.1, 10.0, 100.0]:
            instance.parameters.dt_dia = x | units.yr
            self.assertAlmostEqual(x | units.yr, instance.parameters.dt_dia, in_units=units.yr)
        
        value = instance.get_time()
        self.assertEqual(0.0| units.yr, value)
        self.assertAlmostEqual(0.0 | units.yr, instance.parameters.time, in_units=units.yr)
        for x in [1.0, 10.0, 100.0]:
            instance.parameters.time = x | units.yr
            self.assertAlmostEqual(x | units.yr, instance.parameters.time, in_units=units.yr)
        instance.stop()
    
    def test7(self):
        print("Test7: Testing effect of Hermite parameter epsilon_squared")
        convert_nbody = nbody_system.nbody_to_si(1.0 | units.MSun, 1.0 | units.AU)
        
        particles = datamodel.Particles(2)
        sun = particles[0]
        sun.mass = 1.0 | units.MSun
        sun.position = [0.0, 0.0, 0.0] | units.AU
        sun.velocity = [0.0, 0.0, 0.0] | units.AU / units.yr
        sun.radius = 1.0 | units.RSun

        earth = particles[1]
        earth.mass = 5.9736e24 | units.kg
        earth.radius = 6371.0 | units.km
        earth.position = [0.0, 1.0, 0.0] | units.AU
        earth.velocity = [2.0*numpy.pi, -0.0001, 0.0] | units.AU / units.yr
        
        initial_direction = math.atan((earth.velocity[0]/earth.velocity[1]))
        final_direction = []
        for log_eps2 in range(-9,10,2):
            instance = Hermite(convert_nbody)
            instance.initialize_code()
            instance.parameters.epsilon_squared = 10.0**log_eps2 | units.AU ** 2
            instance.particles.add_particles(particles)
            instance.commit_particles()
            instance.evolve_model(0.25 | units.yr)
            final_direction.append(math.atan((instance.particles[1].velocity[0]/
                instance.particles[1].velocity[1])))
            instance.stop()
        # Small values of epsilon_squared should result in normal earth-sun dynamics: rotation of 90 degrees
        self.assertAlmostEqual(abs(final_direction[0]), abs(initial_direction+math.pi/2.0), 2)
        # Large values of epsilon_squared should result in ~ no interaction
        self.assertAlmostEqual(final_direction[-1], initial_direction, 2)
        # Outcome is most sensitive to epsilon_squared when epsilon_squared = d(earth, sun)^2
        delta = [abs(final_direction[i+1]-final_direction[i]) for i in range(len(final_direction)-1)]
        self.assertEqual(delta[len(final_direction)/2 -1], max(delta))
        
    def test8(self):
        print("Testing Hermite collision_detection")
        particles = datamodel.Particles(7)
        particles.mass = 0.001 | nbody_system.mass
        particles.radius = 0.01 | nbody_system.length
        particles.x = [-101.0, -100.0, -0.5, 0.5, 100.0, 101.0, 104.0] | nbody_system.length
        particles.y = 0 | nbody_system.length
        particles.z = 0 | nbody_system.length
        particles.velocity = [[2, 0, 0], [-2, 0, 0]]*3 + [[-4, 0, 0]] | nbody_system.speed
        
        instance = Hermite()
        instance.initialize_code()
        instance.parameters.set_defaults()
        instance.particles.add_particles(particles)
        collisions = instance.stopping_conditions.collision_detection
        collisions.enable()
        instance.evolve_model(1.0 | nbody_system.time)
        
        self.assertTrue(collisions.is_set())
        self.assertTrue(instance.model_time < 0.5 | nbody_system.time)
        self.assertEqual(len(collisions.particles(0)), 3)
        self.assertEqual(len(collisions.particles(1)), 3)
        self.assertEqual(len(particles - collisions.particles(0) - collisions.particles(1)), 1)
        self.assertEqual(abs(collisions.particles(0).x - collisions.particles(1).x) < 
                (collisions.particles(0).radius + collisions.particles(1).radius),
                [True, True, True])
        
        sticky_merged = datamodel.Particles(len(collisions.particles(0)))
        sticky_merged.mass = collisions.particles(0).mass + collisions.particles(1).mass
        sticky_merged.radius = collisions.particles(0).radius
        for p1, p2, merged in zip(collisions.particles(0), collisions.particles(1), sticky_merged):
            merged.position = (p1 + p2).center_of_mass()
            merged.velocity = (p1 + p2).center_of_mass_velocity()
        
        print(instance.model_time)
        print(instance.particles)
        instance.particles.remove_particles(collisions.particles(0) + collisions.particles(1))
        instance.particles.add_particles(sticky_merged)
        
        instance.evolve_model(1.0 | nbody_system.time)
        print()
        print(instance.model_time)
        print(instance.particles)
        self.assertTrue(collisions.is_set())
        self.assertTrue(instance.model_time < 1.0 | nbody_system.time)
        self.assertEqual(len(collisions.particles(0)), 1)
        self.assertEqual(len(collisions.particles(1)), 1)
        self.assertEqual(len(instance.particles - collisions.particles(0) - collisions.particles(1)), 2)
        self.assertEqual(abs(collisions.particles(0).x - collisions.particles(1).x) < 
                (collisions.particles(0).radius + collisions.particles(1).radius),
                [True])
        instance.stop()
    
    def test9(self):
        convert_nbody = nbody_system.nbody_to_si(1.0 | units.MSun, 149.5e6 | units.km)
    
        instance = Hermite(convert_nbody)
        instance.initialize_code()
        instance.parameters.epsilon_squared = 0.0 | units.AU**2
        instance.parameters.pair_factor = 1e14
        stars = self.new_system_of_sun_and_earth()
        earth = stars[1]
                
        instance.particles.add_particles(stars)
        instance.stopping_conditions.pair_detection.enable()
        instance.evolve_model(365.0 | units.day)
        self.assertTrue(instance.stopping_conditions.pair_detection.is_set())
        instance.particles.copy_values_of_all_attributes_to(stars)
        
        position_at_start = earth.position.value_in(units.AU)[0]
        position_after_full_rotation = earth.position.value_in(units.AU)[0]
        self.assertAlmostEqual(position_at_start, position_after_full_rotation, 6)
    
        print(instance.model_time)
        
        print(instance.stopping_conditions.pair_detection.particles(0).key)
        print(instance.stopping_conditions.pair_detection.particles(1).key)
        
        instance.cleanup_code()
        
        instance.stop()
        
    def test10(self):
        convert_nbody = nbody_system.nbody_to_si(1.0 | units.MSun, 149.5e6 | units.km)
    
        instance = Hermite(convert_nbody)
        instance.initialize_code()
        instance.parameters.epsilon_squared = 0.0 | units.AU**2
        instance.parameters.stopping_conditions_number_of_steps = 10
        self.assertEqual(instance.parameters.stopping_conditions_number_of_steps,10)
    
        stars = self.new_system_of_sun_and_earth()
        earth = stars[1]
                
        instance.particles.add_particles(stars)
        instance.stopping_conditions.number_of_steps_detection.enable()
        instance.evolve_model(365.0 | units.day)
        self.assertTrue(instance.stopping_conditions.number_of_steps_detection.is_set())
        instance.particles.copy_values_of_all_attributes_to(stars)
        
        instance.cleanup_code()
        
        instance.stop()

    def test11(self):
        particles = datamodel.Particles(2)
        particles.x = [0.0,10.0] | nbody_system.length
        particles.y = 0 | nbody_system.length
        particles.z = 0 | nbody_system.length
        particles.radius = 0.005 | nbody_system.length
        particles.vx =  0 | nbody_system.speed
        particles.vy =  0 | nbody_system.speed
        particles.vz =  0 | nbody_system.speed
        particles.mass = 1.0 | nbody_system.mass
       
        instance = Hermite()
        instance.initialize_code()
        instance.parameters.stopping_conditions_number_of_steps = 2
        self.assertEqual(instance.parameters.stopping_conditions_number_of_steps, 2)
        instance.particles.add_particles(particles) 
        instance.stopping_conditions.number_of_steps_detection.enable()
        instance.evolve_model(10 | nbody_system.time)
        self.assertTrue(instance.stopping_conditions.number_of_steps_detection.is_set())
        self.assertTrue(instance.model_time < 10 | nbody_system.time)
        
        instance.stop()

    def test12(self):
        particles = datamodel.Particles(2)
        particles.x = [0.0,1.00] | nbody_system.length
        particles.y = [0.0,0.0] | nbody_system.length
        particles.z = [0.0,0.0] | nbody_system.length
        particles.radius = 0.005 | nbody_system.length
        particles.vx =  [5.1,0.0] | nbody_system.speed
        particles.vy =  [0.0,0.0] | nbody_system.speed
        particles.vz =  [0.0,0.0]| nbody_system.speed
        particles.mass = [0.1,0.1] | nbody_system.mass
       
        instance = Hermite()
        instance.initialize_code()
        instance.parameters.stopping_conditions_out_of_box_size = .5 | nbody_system.length
        self.assertEqual(instance.parameters.stopping_conditions_out_of_box_size, .5 | nbody_system.length)
        instance.particles.add_particles(particles) 
        instance.stopping_conditions.out_of_box_detection.enable()
        instance.evolve_model(0.1 | nbody_system.time)
        self.assertTrue(instance.stopping_conditions.out_of_box_detection.is_set())
        self.assertAlmostEqual(instance.stopping_conditions.out_of_box_detection.particles(0).x, 1.0 |nbody_system.length, 3)
        instance.stop()

    def test13(self):
        particles = plummer.new_plummer_model(31)
       
        instance = Hermite(number_of_workers=1)#, debugger="xterm")
        instance.initialize_code()
        instance.parameters.epsilon_squared = 0.01 | nbody_system.length ** 2
        instance.particles.add_particles(particles)
        
        instance.evolve_model(0.1 | nbody_system.time)
        instance.synchronize_model()
        expected_positions = instance.particles.position
        instance.stop()
        positions_per_workers = []
        for n in [2,3,4,5]:
            instance = Hermite(number_of_workers=n)
            instance.initialize_code()
            instance.parameters.epsilon_squared = 0.01 | nbody_system.length ** 2
            instance.particles.add_particles(particles)
            
            instance.evolve_model(0.1 | nbody_system.time)
            instance.synchronize_model()
            positions_per_workers.append(instance.particles.position)
            instance.stop()
         
         
        for index, n in enumerate([2,3,4,5]):
            self.assertAlmostEqual(expected_positions, positions_per_workers[index], 15)
    
    def test14(self):
        instance = Hermite()
        instance.initialize_code()
        instance.parameters.epsilon_squared = 0.00001 | nbody_system.length**2
        
        particles = datamodel.Particles(2)
        particles.mass = [1.0, 1.0] | nbody_system.mass
        particles.radius =  [0.0001, 0.0001] | nbody_system.length
        particles.position = [[0.0,0.0,0.0], [2.0,0.0,0.0]] | nbody_system.length
        particles.velocity = [[0.0, 0.0, 0.0], [0.0, 0.0, 0.0]] | nbody_system.speed
        instance.particles.add_particles(particles)
        
        zero = 0.0 | nbody_system.length
        fx, fy, fz = instance.get_gravity_at_point(zero, 1.0 | nbody_system.length, zero, zero)
        self.assertAlmostEqual(fx, 0.0 | nbody_system.acceleration, 6)
        self.assertAlmostEqual(fy, 0.0 | nbody_system.acceleration, 6)
        self.assertAlmostEqual(fz, 0.0 | nbody_system.acceleration, 6)

        for x in (0.25, 0.5, 0.75):
            x0 = x | nbody_system.length
            x1 = (2.0 - x) | nbody_system.length
            potential0 = instance.get_potential_at_point(zero, x0, zero, zero)
            potential1 = instance.get_potential_at_point(zero, x1, zero, zero)
            fx0, fy0, fz0 = instance.get_gravity_at_point(zero, x0, zero, zero)
            fx1, fy1, fz1 = instance.get_gravity_at_point(zero, x1, zero, zero)
            
            self.assertAlmostEqual(fy0, 0.0 | nbody_system.acceleration, 6)
            self.assertAlmostEqual(fz0, 0.0 | nbody_system.acceleration, 6)
            self.assertAlmostEqual(fy1, 0.0 | nbody_system.acceleration, 6)
            self.assertAlmostEqual(fz1, 0.0 | nbody_system.acceleration, 6)
            
            self.assertAlmostEqual(fx0, -1.0 * fx1, 5)
            fx = (-1.0 / (x0**2) + 1.0 / (x1**2)) * (1.0 | nbody_system.length ** 3 / nbody_system.time ** 2)
            self.assertAlmostEqual(fx, fx0, 2)
            self.assertAlmostEqual(potential0, potential1, 5)
        instance.stop()
    



import numpy
import time

from amuse.test import amusetest
from amuse.units import units
from amuse.datamodel.memory_storage import InMemoryGridAttributeStorage
from amuse.datamodel.memory_storage import InMemoryVectorQuantityAttribute
        
class TestInMemoryGridAttributeStorage(amusetest.TestCase):
    
    def test1(self):
        x = InMemoryGridAttributeStorage(5,4,3)
        i = (0,1,2,3,4)
        j = (1,3,1,3,1)
        k = (0,2,0,2,0)
        x.set_values_in_store(
            (i,j,k), 
            ['a','b'], 
            [2.0 | units.kg, 1.0 | units.m]
        )
        
        b, a = x.get_values_in_store(None, ['b','a'])
        print(b.shape, a.shape)
        self.assertEqual(b[0][1][0], 1.0 | units.m)
        self.assertEqual(b[0][0][0], 0.0 | units.m)
        self.assertEqual(a[0][1][0], 2.0 | units.kg)
        self.assertEqual(a[1][3][2], 2.0 | units.kg)
        self.assertEqual(a[1][2][2], 0.0 | units.kg)
                
        (b,) = x.get_values_in_store((numpy.s_[0:4], numpy.s_[1:4], numpy.s_[:]), ['a'])
        
        self.assertEqual(b[0][0][0], 2.0 | units.kg)
        self.assertEqual(b[0][0][2], 0.0 | units.kg)
        self.assertEqual(b[1][2][2], 2.0 | units.kg)
        self.assertEqual(b[2][0][0], 2.0 | units.kg)
        self.assertEqual(b[3][2][2], 2.0 | units.kg)
        
        self.assertEqual(b.sum(), 8.0 | units.kg)
        
        self.assertEqual(sorted(x.get_defined_attribute_names()), ["a", "b"])
    
    def test2(self):
        x = InMemoryGridAttributeStorage(5,4,3)
        i = (0,1,2,3,4)
        j = (1,3,1,3,1)
        k = (0,2,0,2,0)
        x.set_values_in_store(
            (i,j,k), 
            ['a','b'], 
            [2.0 , 1.0]
        )
        
        b, a = x.get_values_in_store(None, ['b','a'])
        print(a)
        self.assertEqual(b[0][1][0], 1.0 )
        self.assertEqual(b[0][0][0], 0.0 )
        self.assertEqual(a[0][1][0], 2.0 )
        self.assertEqual(a[1][3][2], 2.0  )
        self.assertEqual(a[1][2][2], 0.0  )
        
        (b,) = x.get_values_in_store((numpy.s_[0:4], numpy.s_[1:4], numpy.s_[:]), ['a'])
        
        self.assertEqual(b[0][0][0], 2.0  )
        self.assertEqual(b[0][0][2], 0.0  )
        self.assertEqual(b[1][2][2], 2.0  )
        self.assertEqual(b[2][0][0], 2.0  )
        self.assertEqual(b[3][2][2], 2.0  )
        
        self.assertEqual(b.sum(), 8.0  )
        
        self.assertEqual(sorted(x.get_defined_attribute_names()), ["a", "b"])
        
        y = x.copy()
        
        (b,) = y.get_values_in_store((numpy.s_[0:4], numpy.s_[1:4], numpy.s_[:]), ['a'])
        
        self.assertEqual(b[0][0][0], 2.0  )
        self.assertEqual(b[0][0][2], 0.0  )
        self.assertEqual(b[1][2][2], 2.0  )
        
        
    def test3(self):
        x = InMemoryGridAttributeStorage(5,4,3)
        i = (0,1,2,3,4)
        j = (1,3,1,3,1)
        k = (0,2,0,2,0)
        x.set_values_in_store(
            (i,j,k), 
            ['a','b'], 
            [2.0 | units.kg, 1.0 | units.m]
        )
        
        b, a = x.get_values_in_store((0,1,0), ['b','a'])
        print(b, a)
        self.assertEqual(b, 1.0 | units.m)
        self.assertEqual(a, 2.0 | units.kg)
        b, a = x.get_values_in_store((0,0,0), ['b','a'])
        print(b, a)
        self.assertEqual(b, 0.0 | units.m)
        self.assertEqual(a, 0.0 | units.kg)
                
        
class TestInMemoryVectorQuantityAttribute(amusetest.TestCase):
    
    def test1(self):
        quantity  = units.m.new_quantity(numpy.array([1.0,2.0,3.0]))
        attribute = InMemoryVectorQuantityAttribute('test', quantity.shape, quantity.unit)
        attribute.set_values(None, quantity)
        
        self.assertEqual(attribute.get_length(), 3)
        self.assertEqual(attribute.get_shape(), (3,) )
        self.assertEqual(attribute.get_values([1,2]), [2.0,3.0] | units.m)
    
        attribute.increase_to_length(5)
        self.assertEqual(attribute.get_values(None), [1.0,2.0,3.0,0.0,0.0] | units.m)
        
    def test2(self):
        quantity  = units.m.new_quantity(numpy.array([1.0,2.0,3.0]))
        attribute = InMemoryVectorQuantityAttribute('test', quantity.shape, quantity.unit)
        attribute.set_values(None, quantity)
        attribute.set_values([1,2], [4.0,5.0] | units.m)
    
        attribute.increase_to_length(5)
        self.assertEqual(attribute.get_values(None), [1.0,4.0,5.0,0.0,0.0] | units.m)

    def test3(self):
        quantity  = units.m.new_quantity(numpy.array([[1.0,2.0,3.0], [4.0,5.0,6.0]]))
        attribute = InMemoryVectorQuantityAttribute('test', quantity.shape, quantity.unit)
        attribute.set_values(None, quantity)
        self.assertEqual( attribute.get_values([1]),  [4.0,5.0,6.0] | units.m)
        self.assertEqual( attribute.get_shape(), (2,3))
        attribute.increase_to_length(4)
        self.assertEqual( attribute.get_shape(), (4,3))
        self.assertEqual(attribute.get_values(None), [[1.0,2.0,3.0], [4.0,5.0,6.0], [0.0,0.0,0.0], [0.0,0.0,0.0]] | units.m)




from amuse.test import amusetest
import os.path
import shutil
import subprocess
import sys

from amuse.rfi.tools import create_dir

class CreateADirectoryAndPopulateItWithFilesForACommunityCodeTest(amusetest.TestWithMPI):
    
    def setUp(self):
        
        self.check_can_compile_modules()
        
        
    def test1(self):
        instance = create_dir.CreateADirectoryAndPopulateItWithFilesForACCode()
        instance.name_of_the_code_interface_class = 'TestCode'
        self.assertEqual(instance.name_of_the_community_code, 'testcode')
        self.assertTrue(instance.path_of_the_community_code.endswith('testcode'))
            
    def test2(self):
        root = self.get_path_to_results()
        working_dir = os.path.join(root, 'testcode')
        
        print(working_dir)
        
        if os.path.exists(working_dir):
            shutil.rmtree(working_dir)
                        
        instance = create_dir.CreateADirectoryAndPopulateItWithFilesForACCode()
        instance.name_of_the_code_interface_class = 'TestCode'
        instance.path_of_the_root_directory = root
        #~ if create_dir.CreateADirectoryAndPopulateItWithFiles().amuse_root_dir == os.getcwd():
            #~ self.assertEqual(instance.reference_to_amuse_path,'../..')
        
        instance.start()
        
        self.assertTrue(os.path.exists(working_dir))
        self.assertTrue(os.path.exists(os.path.join(working_dir,'interface.py')))
        self.assertTrue(os.path.exists(os.path.join(working_dir,'Makefile')))
        
        
        
        call = subprocess.Popen(
            'make',
            cwd=working_dir, 
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
        )
        outputstring, errorstring = call.communicate()
        
        
        self.assertEqual(call.returncode, 0)
        
        self.assertTrue(os.path.exists(os.path.join(working_dir,'testcode_worker')))
        
        sys.path.insert(0, root)
        
        try:
            __import__('testcode.interface')
        except:
            self.fail("import of code failed")
        
        module = sys.modules['testcode.interface']
        instance = module.TestCode()
        result = instance.echo_int(12)
        
        self.assertEqual(result, 12)
        
        instance.stop()
        
        del sys.path[0]
        
        self.assertTrue(os.path.exists(os.path.join(working_dir,'worker_code.cc')))
        call = subprocess.Popen(
            ['make', 'clean'],
            cwd=working_dir, 
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
        )
        outputstring, errorstring = call.communicate()
        
        print(errorstring)
        self.assertFalse(os.path.exists(os.path.join(working_dir,'worker_code.cc')))
    
        self.assertEqual(call.returncode, 0)
        
        if os.path.exists(working_dir):
            shutil.rmtree(working_dir)
        
    def test3(self):
        root = self.get_path_to_results()
        working_dir = os.path.join(root, 'testcodef')
        
        print(working_dir)
        
        if os.path.exists(working_dir):
            shutil.rmtree(working_dir)
                        
        instance = create_dir.CreateADirectoryAndPopulateItWithFilesForAFortranCode()
        instance.name_of_the_code_interface_class = 'TestCodeF'
        instance.path_of_the_root_directory = root
        #~ if create_dir.CreateADirectoryAndPopulateItWithFiles().amuse_root_dir == os.getcwd():
            #~ self.assertEqual(instance.reference_to_amuse_path,'../..')
        
        instance.start()
        
        self.assertTrue(os.path.exists(working_dir))
        self.assertTrue(os.path.exists(os.path.join(working_dir,'interface.py')))
        self.assertTrue(os.path.exists(os.path.join(working_dir,'Makefile')))
        
        call = subprocess.Popen(
            'make',
            cwd=working_dir, 
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
        )
        outputstring, errorstring = call.communicate()
        
        print(errorstring)
    
        self.assertEqual(call.returncode, 0)
        
        self.assertTrue(os.path.exists(os.path.join(working_dir,'testcodef_worker')))
        
        
        sys.path.insert(0, root)
        
        try:
            __import__('testcodef.interface')
        except:
            self.fail("import of code failed")
        
        module = sys.modules['testcodef.interface']
        instance = module.TestCodeF()
        result = instance.echo_int(12)
        
        self.assertEqual(result, 12)
        
        instance.stop()
        
        del sys.path[0]
        
        self.assertTrue(os.path.exists(os.path.join(working_dir,'worker_code.f90')))
        
        call = subprocess.Popen(
            ['make', 'clean'],
            cwd=working_dir, 
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
        )
        outputstring, errorstring = call.communicate()
        print(errorstring)
        
        self.assertFalse(os.path.exists(os.path.join(working_dir,'worker_code.f90')))
    
        self.assertEqual(call.returncode, 0)
        
        
   #     if os.path.exists(working_dir):
        #    shutil.rmtree(working_dir)
        
        
            
        



from amuse.test import amusetest
import numpy

from amuse.support.exceptions import AmuseException



from amuse.units import core
from amuse.units.units import *
from amuse.units.constants import *
class TestUnitConversions(amusetest.TestCase):
    def test1(self):
        km = 1000 * m
        self.assertEqual(1000, km.value_in(m))
        self.assertEqual(0.001, m.value_in(km))
        
    def test2(self):
        km = 1000 * m
        val = km(10)
        self.assertEqual(10000, val.value_in(m))
        
    def test3(self):
        km = 1000.0 * m
        val = km(10)
        self.assertEqual("10 1000.0 * m", str(val))
        
        km = named('kilometer','km',1000 * m)
        val = km(10)
        self.assertEqual("10 km", str(val))
        
    def test4(self):
        km = named('kilometer','km',1000 * m)
        h = named('hour','h',60 * 60 * s)
        kmh = km/h
        ms = m/s
        val = 10 | m/s
        self.assertEqual(36,val.value_in(kmh))
        
    def test5(self):
        km = named('kilometer','km',1000 * m)
        h = named('hour','h',60 * 60 * s)
        kmh = km/h
        ms = m/s
        val = 10 | ms
        self.assertEqual(36,val.value_in(kmh))
        
    def test6(self):
        no1 = m / m
        no2 = no_unit
        self.assertEqual("m / m", str(no1))
        self.assertTrue(no1.has_same_base_as(no2))

        
    def test7(self):
        x = (100 * kg ** 2)  / kg 
        self.assertEqual("100 * kg**2 / kg", str(x))
        self.assertEqual("100.0 * kg", str(x.to_simple_form()))
        
        
    def test8(self):
        x = (10 | g) * (2 | m) 
        self.assertEqual("20 0.001 * m * kg", str(x))
        self.assertEqual("0.02 kg * m", str(x.as_quantity_in(kg * m)))
        x = (10 | kg) * (2000 | g) 
        self.assertEqual("20000 0.001 * kg**2", str(x))
        
    def test9(self):
        speed_of_light = 1 | (lightyear * yr**-1)
        time = 1e-9 | s
        length = speed_of_light * time
        length_in_m = length.value_in(m)
        self.assertAlmostEqual(0.2997988, length_in_m, 6)
        
    def test10(self):
        eps0_1 = mu0**-1*c**-2
        eps0_2 = (1 | none)/(mu0*(c**2))
        self.assertTrue((eps0_1 - eps0_2) / eps0_1 < (1e-10 | none))
        b =((1.|e)**2)
        f = (hbar*c*4.*numpy.pi* eps0)**-1
        fine_structure_constant_calculated = (b * f)
        fine_structure_constant = 7.297352537650e-3
        self.assertAlmostEqual(fine_structure_constant_calculated, fine_structure_constant, 5)
        
    

    def test11(self):
        vel1 = 1 | m / s
        self.assertRaises(core.IncompatibleUnitsException, vel1.as_quantity_in, s / m, 
            expected_message = "Cannot express m / s in s / m, the units do not have the same bases")
    
    def test12(self):
        self.assertEqual((1234 | g).as_string_in(g), '1234 g')
        self.assertEqual((1234 | g).as_string_in(kg), '1.234 kg')
        self.assertEqual((1.234 | kg).as_string_in(g), '1234.0 g')
        self.assertEqual((1.0 | km * s**-1).as_string_in(m / s), '1000.0 m / s')
        self.assertEqual((1.0 | km * s**-1).as_string_in(s**-1 * m), '1000.0 s**-1 * m')
        self.assertEqual((1.0 | km / s).as_string_in((10*J/g)**0.5), '10.0 (10 * J / g)**0.5')

    def test13(self):
        self.assertAlmostRelativeEquals((1234 | km ** (5/3.)).value_in( (km ** 5.0) ** (1.0/3.0) ), 1234.0, 12)
        self.assertAlmostRelativeEquals((1234 | km ** (5/3.)).value_in( (m ** 5.0) ** (1.0/3.0) ), 123400000.0)
        
    def test15(self):
        self.assertAlmostRelativeEquals( (m / s).to_array_of_floats(), [1, 1,  1, 0, -1, 0, 0, 0, 0])
        self.assertAlmostRelativeEquals( (s / m).to_array_of_floats(), [1, 1, -1, 0, 1, 0, 0, 0, 0])
        self.assertAlmostRelativeEquals( (kg ** 2/ s).to_array_of_floats(), [1, 1,  0, 2, -1, 0, 0, 0, 0])
        


class TestNonNumericUnits(amusetest.TestCase):
    def test1(self):
        string1 = "string" | string
        self.assertRaises(AmuseException, string1.as_quantity_in, m, 
            expected_message = "Cannot convert non-numeric quantities in to another unit")

    def test2(self):
        x = "test" | string
        self.assertEqual("test", x.value_in(string))  
            
    def test3(self):
        test_unit = core.enumeration_unit(
            "test", 
            "test",
            [1,2,3],
            ["one", "two", "three"]
        )
        x = 1 | test_unit
        self.assertEqual(1, x.value_in(test_unit))    
        self.assertEqual("one", str(x))  
        self.assertRaises(Exception, lambda: 4 | test_unit, 
            expected_message = "<4> is not a valid value for unit<test>")
    
    def test4(self):
        self.assertRaises(Exception, lambda: 1 | string, 
            expected_message = "<1> is not a valid value for unit<string>")
        
    def test5(self):
        test_unit = core.enumeration_unit(
            "test", 
            "test",
            [1,2,3],
            ["one", "two", "three"]
        )
        self.assertEqual(3, len(list(test_unit.quantities())))  
        for x, y in  zip(test_unit.quantities(), ["one", "two", "three"]):
            self.assertEqual(str(x), y)
    
       
    def test6(self):
        test_unit = core.enumeration_unit(
            "test", 
            "test",
            [1,4,7]
        )
        self.assertEqual(3, len(list(test_unit.quantities())))  
        for x, y in  zip(test_unit.quantities(), ["1", "4", "7"]):
            self.assertEqual(str(x), y)
    
    def test7(self):
        test_unit = core.enumeration_unit(
            "test", 
            "test",
            list(range(5))
        )
        self.assertEqual(5, len(list(test_unit.quantities())))
        self.assertEqual(1 | test_unit, 1 | test_unit)
        self.assertTrue (1 | test_unit == 1 | test_unit)
        self.assertFalse(1 | test_unit == 2 | test_unit)
        self.assertTrue (1 | test_unit != 2 | test_unit)
        self.assertFalse(1 | test_unit != 1 | test_unit)
        self.assertTrue (1 | test_unit >= 1 | test_unit)
        self.assertFalse(1 | test_unit >= 2 | test_unit)
        self.assertTrue (1 | test_unit <= 1 | test_unit)
        self.assertFalse(1 | test_unit <= 0 | test_unit)
        self.assertTrue (1 | test_unit >  0 | test_unit)
        self.assertFalse(1 | test_unit >  1 | test_unit)
        self.assertTrue (1 | test_unit <  3 | test_unit)
        self.assertFalse(1 | test_unit <  0 | test_unit)
        
    



from amuse.support.interface import InCodeComponentImplementation

from amuse.test.amusetest import TestWithMPI
from amuse.test.amusetest import TestCase
from amuse.test import compile_tools
from amuse.support import exceptions
from amuse.support import options

import os
import time
import sys


from amuse.units import nbody_system
from amuse.units import units
from amuse import datamodel
from amuse.rfi.tools import create_c
from amuse.rfi import channel
from amuse.rfi.core import *

from . import test_c_implementation
from amuse.rfi.tools import create_cython

codestring = """
#include <stdio.h>
#include <stdlib.h>

int echo_int(int int_in, int * int_out) {
    *int_out = int_in;
    if(int_in < 0) {
        return -1;
    } else {
        return 0;
    }
}"""


import shlex
from amuse.test.amusetest import get_amuse_root_dir

from . import test_fortran_implementation

from amuse.rfi.tools import create_fortran

class ForTestingInterface(CodeInterface):
    
    def __init__(self, exefile, **options):
        CodeInterface.__init__(self, exefile, **options)

    @legacy_function
    def echo_int():
        function = LegacyFunctionSpecification()
        function.addParameter('int_in', dtype='int32', direction=function.IN)
        function.addParameter('int_out', dtype='int32', direction=function.OUT)
        function.result_type = 'int32'
        function.can_handle_array = True
        return function 
            
    
class ForTesting(InCodeComponentImplementation):
    
    def __init__(self, exefile, **options):
        InCodeComponentImplementation.__init__(self, ForTestingInterface(exefile, **options), **options)
    
    def define_methods(self, object):
        object.add_method(
            'echo_int',
            (units.m,)
            ,
            (
                units.m,
                object.ERROR_CODE,
            )
        )

            
class TestCreateCython(TestCase):
    
    def test1(self):
        uc = create_cython.GenerateACythonSourcecodeStringFromASpecificationClass()
        uc.specification_class = test_c_implementation.ForTestingInterface
        uc.needs_mpi = True
        code =  uc.result
        #print code
        self.assertTrue('cdef extern from "worker_code.h":' in code)
        self.assertTrue('int c_echo_int "echo_int" (int, int *)' in code)
        

    def test2(self):
        uc = create_cython.GenerateAFortranInterfaceStringOfAFunctionSpecification()
        uc.specification = test_c_implementation.ForTestingInterface.echo_int.specification
        uc.needs_mpi = True
        uc.start()
        code =  uc.result
        print('<c>' + code + '</c>')
        self.assertTrue('INTEGER(kind = c_int) function c_3f33a9ce(int_in, &\n &int_out) &\n & result(rrreeesss) &\n & bind(c, name = "ci_echo_int")' in code)
        self.assertTrue('  INTEGER(kind = c_int), intent(in), value :: int_in' in code)
        self.assertTrue('  INTEGER(kind = c_int), intent(out) :: int_out' in code)
        self.assertTrue('  INTEGER :: echo_int' in code)
        self.assertTrue('  rrreeesss = echo_int(int_in, &\n &int_out)' in code)
        self.assertTrue('END FUNCTION c_3f33a9ce' in code)
        



    def test3(self):
        uc = create_cython.GenerateAFortranInterfaceStringOfAFunctionSpecification()
        uc.specification = test_c_implementation.ForTestingInterface.echo_string.specification
        uc.needs_mpi = True
        uc.start()
        code =  uc.result
        print('<c>' + code + '</c>')
        self.assertTrue('INTEGER(kind = c_int) function c_64452f1a(string_in, &\n &string_out) &\n & result(rrreeesss) &\n & bind(c, name = "ci_echo_string")' in code)
        self.assertTrue('  type(C_ptr), intent(in), value :: string_in' in code)
        self.assertTrue('  character(len=4096) :: string_string_in' in code)
        self.assertTrue('  call C_F_string_ptr(string_in, string_string_in)' in code)
        self.assertTrue('  call C_F_string_ptr(string_in, string_string_in)' in code)
        self.assertTrue('  rrreeesss = echo_string(string_string_in, &\n &string_string_out)' in code)
        self.assertTrue('     string_buffer1 = C_malloc(sz)' in code)
        self.assertTrue('  call F_C_string_ptr(trim(string_string_out), string_buffer1, 4096)' in code)
        self.assertTrue('END FUNCTION c_64452f1a' in code)
        




class TestCythonImplementationInterface(test_c_implementation.TestCImplementationInterface):


    @classmethod
    def setup_class(cls):
        cls.skip_if_no_cython()
        cls.check_can_compile_modules()
        try:
            cls.build_worker()
        except Exception as ex:
            print(ex)
            raise

    @classmethod
    def teardown_class(cls):
        pass

    def setUp(self):
        super(test_c_implementation.TestCImplementationInterface, self).setUp()

    def test22(self):
        self.skip("this test uses mpi internals, skip here")
                     
    @classmethod
    def skip_if_no_cython(cls):
        
        if sys.hexversion > 0x03000000:
            cls.skip("no cython for python 3.0")

        if not hasattr(config.compilers, 'cython') or len(config.compilers.cython) == 0:
            cls.skip("cython not configured")
            
        process, output, error = compile_tools.open_subprocess([config.compilers.cython, '-V'])
        if process.returncode:
            cls.skip("cython not available")
        
        error = error.strip()[0:7].decode('utf-8')
        if not error.startswith('Cython'):
            cls.skip("cython not available")
        
            
        
        
            
    @classmethod
    def build_worker(cls):
        
        path = os.path.abspath(cls.get_path_to_results())
        codefile = os.path.join(path,"code.o")
        interfacefile = os.path.join(path,"interface.o")
        headerfile = os.path.join(path,"worker_code.h")
        cls.sofile = os.path.join(path,"interface.so")
        cls.exefile = os.path.join(path,"c_worker")
        
        compile_tools.c_compile(codefile, test_c_implementation.codestring,
                                extra_args=["-fPIC"])
        
        uc = create_c.GenerateACHeaderStringFromASpecificationClass()
        uc.specification_class = test_c_implementation.ForTestingInterface
        uc.needs_mpi = False
        header =  uc.result
        
        
        with open(headerfile, "w") as f:
            f.write(header)
        
        
        root, ext = os.path.splitext(interfacefile)
        sourcename = root + '.pyx'
        cname = root + '.c'
        
        uc = create_cython.GenerateACythonSourcecodeStringFromASpecificationClass()
        uc.specification_class = test_c_implementation.ForTestingInterface
        uc.needs_mpi = True
        code =  uc.result

        


        
        with open(sourcename, "w") as f:
            f.write(code)


        uc = create_cython.GenerateACythonStartScriptStringFromASpecificationClass()
        uc.specification_class = test_c_implementation.ForTestingInterface
        uc.needs_mpi = True
        script =  uc.result
        
        with open(cls.exefile, "w") as f:
            f.write(script)

        os.chmod(cls.exefile, 0o777)
        
        import mpi4py
        process, stdout, stderr = compile_tools.open_subprocess([config.compilers.cython, 
        '-I',
        mpi4py.get_include(),
         sourcename, '-o', cname])

        
        if process.returncode == 0:
            compile_tools.wait_for_file(cname)
        
        if process.returncode != 0 or not os.path.exists(cname):
            print("Could not cythonize {0}, error = {1}".format(sourcename, stderr))
            raise Exception("Could not cythonize {0}, error = {1}".format(sourcename, stderr))
        
        with open(cname, "r") as f:
            string = f.read()
            
        compile_tools.c_pythondev_compile(interfacefile, string)
        # cls.c_pythondev_build(cls.exefile, [interfacefile, codefile] )
        compile_tools.c_pythondev_buildso(cls.sofile,  [interfacefile, codefile] )

    def test1(self):
        instance = ForTestingInterface(self.exefile)
        int_out, error = instance.echo_int(10)
        instance.stop()
        self.assertEqual(int_out, 10)
        self.assertEqual(error, 0)
        
    def test14(self):
        self.skip("needs python support")


class TestCythonFortranImplementationInterface(test_fortran_implementation.TestInterface):

    @classmethod
    def setup_class(cls):
        cls.skip_if_no_cython()
        print("building")
        cls.check_can_compile_modules()
        cls.build_worker()

    @classmethod
    def teardown(cls):
        pass

    def setUp(self):
        super(test_fortran_implementation.TestInterface, self).setUp()

    def test22(self):
        self.skip("this test uses mpi internals, skip here")

    @classmethod
    def skip_if_no_cython(cls):

        if sys.hexversion > 0x03000000:
            cls.skip("no cython for python 3.0")
        
        if not hasattr(config.compilers, 'cython') or len(config.compilers.cython) == 0:
            cls.skip("cython not configured")
            
        process, output, error = compile_tools.open_subprocess([config.compilers.cython, '-V'])
        if process.returncode:
            cls.skip("cython not available")
        
        error = error.strip()[0:7].decode('utf-8')
        if not error.startswith('Cython'):
            cls.skip("cython not available")

    @classmethod
    def build_worker(cls):
        
        path = os.path.abspath(cls.get_path_to_results())
        codefile = os.path.join(path,"code.o")
        interfacefile = os.path.join(path,"interface.o")
        headerfile = os.path.join(path,"worker_code.h")
        cls.sofile = os.path.join(path,"interface.so")
        cls.interfacec_o_file = os.path.join(path,"interfacec.o")
        cls.exefile = os.path.join(path,"c_worker")
        
        compile_tools.fortran_compile(codefile, test_fortran_implementation.codestring,
                                      extra_args=["-fPIC"])
        
        
        uc = create_c.GenerateACHeaderStringFromASpecificationClass()
        uc.specification_class = test_fortran_implementation.ForTestingInterface
        uc.needs_mpi = False
        header =  uc.result
        
        
        with open(headerfile, "w") as f:
            f.write(header)
        
        
        root, ext = os.path.splitext(interfacefile)
        sourcename = root + '.pyx'
        cname = root + '.c'
        
        uc = create_cython.GenerateACythonSourcecodeStringFromASpecificationClass()
        uc.specification_class = test_fortran_implementation.ForTestingInterface
        uc.function_name_prefix = "ci_"
        uc.needs_mpi = True
        code =  uc.result
        
        with open(sourcename, "w") as f:
            f.write(code)


        uc = create_cython.GenerateACythonStartScriptStringFromASpecificationClass()
        uc.specification_class = test_fortran_implementation.ForTestingInterface
        uc.needs_mpi = True
        script =  uc.result
        
        with open(cls.exefile, "w") as f:
            f.write(script)

        os.chmod(cls.exefile, 0o777)

        import mpi4py
        process, stdout, stderr = compile_tools.open_subprocess([config.compilers.cython, 
        '-I',
        mpi4py.get_include(),
         sourcename, '-o', cname])

        if process.returncode == 0:
            compile_tools.wait_for_file(cname)
        
        if process.returncode != 0 or not os.path.exists(cname):
            print("Could not cythonize {0}, error = {1}".format(sourcename, stderr))
            raise Exception("Could not cythonize {0}, error = {1}".format(sourcename, stderr))
        
        with open(cname, "r") as f:
            string = f.read()


        
        
        uc = create_cython.GenerateAFortranInterfaceSourcecodeStringFromASpecificationClass()
        uc.specification_class = test_fortran_implementation.ForTestingInterface
        uc.function_name_prefix = "ci_"
        uc.needs_mpi = False
        code =  uc.result

        compile_tools.fortran_compile(cls.interfacec_o_file, code,
                                      extra_args=["-fPIC"])

        compile_tools.c_pythondev_compile(interfacefile, string)
        compile_tools.fortran_pythondev_buildso(cls.sofile,  [interfacefile, codefile, cls.interfacec_o_file] )

    def test1(self):
        instance = ForTestingInterface(self.exefile)
        int_out, error = instance.echo_int(10)
        instance.stop()
        self.assertEqual(int_out, 10)
        self.assertEqual(error, 0)



from amuse.test import amusetest
from io import StringIO
import textwrap
import os
import numpy


from amuse import io
from amuse.io import vtk
from amuse.units import units
from amuse.units import generic_unit_system
from amuse.units import generic_unit_system
from amuse import datamodel

class VtkStructuredGridTests(amusetest.TestCase):
    
    def test1(self):
        grid = datamodel. new_regular_grid([2,3,4], [1,1,1] | generic_unit_system.length)
        grid.rho = grid.x * (0.1 | generic_unit_system.mass / generic_unit_system.length ** 4)
        data_file = StringIO()
        instance = vtk.VtkStructuredGrid("test.vts", data_file, grid)
        instance.store()
        
        contents = data_file.getvalue()
        self.assertTrue(contents.find('WholeExtent="0 2 0 3 0 4"')> 0)
        self.assertTrue(contents.find('<DataArray type="Float64" NumberOfComponents="3">')> 0)
        self.assertTrue(contents.find('<DataArray type="Float64" NumberOfComponents="1" Name="rho">')> 0)
    

    def test2(self):
        grid = datamodel. new_regular_grid([4,5,6], [1,1,1] | generic_unit_system.length)
        grid.mass = generic_unit_system.density(numpy.random.rand(4,5,6))
        data_file = StringIO()
        instance = vtk.VtkStructuredGrid("test.vts", data_file, grid)
        instance.store()
        
        contents = data_file.getvalue()
        self.assertTrue(contents.find('<Piece Extent="0 4 0 5 0 6">')> 0)
        self.assertTrue(contents.find('<CellData>')> 0)
        self.assertTrue(contents.find('<PointData>')> 0)
        self.assertTrue(contents.find('<DataArray type="Float64" NumberOfComponents="3">')> 0)
        self.assertTrue(contents.find('<DataArray type="Float64" NumberOfComponents="1" Name="mass">')> 0)
    
class VtkUnstructuredGridTests(amusetest.TestCase):
    
    def test1(self):
        grid = datamodel. new_regular_grid([2,3,4], [1,1,1] | generic_unit_system.length)
        grid.rho = grid.x * (0.1 | generic_unit_system.mass / generic_unit_system.length ** 4)
        data_file = StringIO()
        instance = vtk.VtkUnstructuredGrid("test.vtu", data_file, grid)
        instance.store()
        contents = data_file.getvalue()
        
        self.assertTrue(contents.find('<Piece NumberOfPoints="60" NumberOfCells="24"')> 0)
        self.assertTrue(contents.find('DataArray type="Int32" NumberOfComponents="1" Name="connectivity">')> 0)
        self.assertTrue(contents.find('DataArray type="Int32" NumberOfComponents="1" Name="offsets">')> 0)
        self.assertTrue(contents.find('11 11 11')> 0)
        self.assertTrue(contents.find('8 16 24')> 0)
        self.assertTrue(contents.find('DataArray type="Float64" NumberOfComponents="3" Name="points">')> 0)
        
    
    def test2(self):
        grid = datamodel. new_regular_grid([2,2,2], [1,1,1] | generic_unit_system.length)
        grid.rho = grid.x * (0.1 | generic_unit_system.mass / generic_unit_system.length ** 4)
        data_file = StringIO()
        instance = vtk.VtkUnstructuredGrid("test.vtu", data_file, grid)
        instance.store()
        contents = data_file.getvalue()
        
        self.assertTrue(contents.find('<Piece NumberOfPoints="27" NumberOfCells="8"')> 0)
        self.assertTrue(contents.find('DataArray type="Int32" NumberOfComponents="1" Name="connectivity">')> 0)
        self.assertTrue(contents.find('DataArray type="Int32" NumberOfComponents="1" Name="offsets">')> 0)
        self.assertTrue(contents.find('11 11 11')> 0)
        self.assertTrue(contents.find('56 64')> 0)
        self.assertTrue(contents.find('DataArray type="Float64" NumberOfComponents="3" Name="points">')> 0)
        self.assertTrue(contents.find('<DataArray type="Float64" NumberOfComponents="1" Name="rho">')>0)
        
    
    def test3(self):
        grid1 = datamodel. new_regular_grid([2,2,2], [1,1,1] | generic_unit_system.length)
        grid1.rho = grid1.x * (0.1 | generic_unit_system.mass / generic_unit_system.length ** 4)
        grid2 = datamodel. new_regular_grid([2,2,2], [1,1,1] | generic_unit_system.length)
        grid2.position += [1.0,0.0,0.0] | generic_unit_system.length
        grid2.rho = grid2.x * (0.1 | generic_unit_system.mass / generic_unit_system.length ** 4)
        
        data_file = StringIO()
        instance = vtk.VtkUnstructuredGrid("test.vtu", data_file, [grid1, grid2])
        instance.store()
        contents = data_file.getvalue()
        index_of_piece_1 = contents.find('<Piece NumberOfPoints="27" NumberOfCells="8"')
        index_of_piece_2 = contents.find('<Piece NumberOfPoints="27" NumberOfCells="8"', index_of_piece_1 + 1)
        self.assertTrue(index_of_piece_1 > 0)
        self.assertTrue(index_of_piece_2 > 0)
        self.assertTrue(contents.find('DataArray type="Int32" NumberOfComponents="1" Name="connectivity">')> 0)
        self.assertTrue(contents.find('DataArray type="Int32" NumberOfComponents="1" Name="offsets">')> 0)
        self.assertTrue(contents.find('11 11 11')> 0)
        self.assertTrue(contents.find('56 64')> 0)
        self.assertTrue(contents.find('DataArray type="Float64" NumberOfComponents="3" Name="points">')> 0)
        self.assertTrue(contents.find('<DataArray type="Float64" NumberOfComponents="1" Name="rho">')>0)
        
